Package: ICCDesign
Title: Intraclass Correlation Coefficient (ICC) Design, Calculation and
        Interactive 'shiny' Toolkit
Version: 0.1.1
Authors@R: c(
    person("Ziyu", "Liu", 
           email = "1755454769@qq.com",
           role = c("aut", "cre")),
    person("Ruilin", "Ma", 
           role = "aut"),
    person("Chenge", "Gao", 
           role = "aut"),
    person("Yundan", "Zhang", 
           role = "aut")
  )
Description: A comprehensive toolkit for intraclass correlation coefficient (ICC) analysis,
    integrating three core functionalities: (1) Closed-form sample size calculation for ICC
    estimation with assurance probability, based on Zou (2012) <doi:10.1002/sim.5466>;
    (2) Full implementation of all 10 ICC types (6 common + 4 supplementary) for point
    estimation, exact confidence interval calculation, and formal hypothesis testing,
    following the methods of McGraw & Wong (1996) <doi:10.1037/1082-989X.1.1.30> and
    the standard decision framework; (3) An interactive 'shiny' application that guides users
    through ICC type selection, performs calculations, and provides reliability evaluation
    based on the Koo & Li (2016) <doi:10.1016/j.jcm.2016.02.012> criteria.
    Compared to existing packages, it provides a unified decision workflow and supports
    all less common ICC variants.
License: GPL (>= 3)
Depends: R (>= 4.1.0)
Imports: stats, shiny
Suggests: testthat, knitr, rmarkdown
Encoding: UTF-8
LazyData: false
URL: https://github.com/KlariZhang/ICCDesign
BugReports: https://github.com/KlariZhang/ICCDesign/issues
Config/roxygen2/version: 8.0.0
RoxygenNote: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-02 10:25:35 UTC; Lenovo
Author: Ziyu Liu [aut, cre],
  Ruilin Ma [aut],
  Chenge Gao [aut],
  Yundan Zhang [aut]
Maintainer: Ziyu Liu <1755454769@qq.com>
Repository: CRAN
Date/Publication: 2026-06-02 11:00:13 UTC
Built: R 4.6.0; ; 2026-06-02 11:47:09 UTC; unix

Package: ICCDesign
Title: Intraclass Correlation Coefficient (ICC) Design, Calculation and
        Interactive 'shiny' Toolkit
Version: 0.1.1
Authors@R: c(
    person("Ziyu", "Liu", 
           email = "1755454769@qq.com",
           role = c("aut", "cre")),
    person("Ruilin", "Ma", 
           role = "aut"),
    person("Chenge", "Gao", 
           role = "aut"),
    person("Yundan", "Zhang", 
           role = "aut")
  )
Description: A comprehensive toolkit for intraclass correlation coefficient (ICC) analysis,
    integrating three core functionalities: (1) Closed-form sample size calculation for ICC
    estimation with assurance probability, based on Zou (2012) <doi:10.1002/sim.5466>;
    (2) Full implementation of all 10 ICC types (6 common + 4 supplementary) for point
    estimation, exact confidence interval calculation, and formal hypothesis testing,
    following the methods of McGraw & Wong (1996) <doi:10.1037/1082-989X.1.1.30> and
    the standard decision framework; (3) An interactive 'shiny' application that guides users
    through ICC type selection, performs calculations, and provides reliability evaluation
    based on the Koo & Li (2016) <doi:10.1016/j.jcm.2016.02.012> criteria.
    Compared to existing packages, it provides a unified decision workflow and supports
    all less common ICC variants.
License: GPL (>= 3)
Depends: R (>= 4.1.0)
Imports: stats, shiny
Suggests: testthat, knitr, rmarkdown
Encoding: UTF-8
LazyData: false
URL: https://github.com/KlariZhang/ICCDesign
BugReports: https://github.com/KlariZhang/ICCDesign/issues
Config/roxygen2/version: 8.0.0
RoxygenNote: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-02 10:25:35 UTC; Lenovo
Author: Ziyu Liu [aut, cre],
  Ruilin Ma [aut],
  Chenge Gao [aut],
  Yundan Zhang [aut]
Maintainer: Ziyu Liu <1755454769@qq.com>
Repository: CRAN
Date/Publication: 2026-06-02 11:00:13 UTC
Built: R 4.6.0; ; 2026-06-02 13:48:44 UTC; unix

# Generated by roxygen2: do not edit by hand

export(icc_calc)
export(icc_power)
export(icc_preprocess_data)
export(icc_sample_size)
export(icc_sample_size_lower)
export(icc_sample_size_width)
export(run_icc_app)

# ICCDesign

A comprehensive R package for intraclass correlation coefficient (ICC) analysis and sample size planning.

## Features

- Full support for all 10 ICC types from McGraw & Wong (1996)
- Intuitive 4-question decision framework (no need to memorize ICC codes)
- Automated reliability evaluation based on Koo & Li (2016)
- Publication-ready report generation
- Sample size and power analysis
- Interactive Shiny web application with visualization
- Comprehensive data validation and error handling

## Installation

```r
# Install devtools if not already installed
install.packages("devtools")

# Install ICCDesign from GitHub
devtools::install_github("KlariZhang/ICCDesign")

# Load the package
library(ICCDesign)
```

## Quick Start

1. Command Line Interface

```r
# Use the built-in example dataset
data(icc_data)

# Calculate ICC (two-way random, single rating, absolute agreement)
result <- icc_calc(
  data = icc_data,
  same_raters = TRUE,
  rater_effect = "random",
  rating_type = "single",
  agreement_type = "absolute"
)

# View the full report
print(result)

# Extract specific results
result$icc_result$point_est  # ICC point estimate
result$evaluation$rating_en  # Reliability rating
```

2. Interactive Shiny Interface

```r
### Launch the point-and-click web application
run_icc_app() 3. Sample Size Calculation
```

3. Sample Size Calculation

```r
### Calculate required sample size for 80% assurance that ICC >= 0.75

n <- icc_sample_size(
method = "lower",
rho = 0.85,
rating_target = "good",
k = 3,
same_raters = TRUE,
rater_effect = "random",
rating_type = "single",
agreement_type = "absolute"
)

cat("Required sample size:", n, "\n")
```

References

1. McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
2. Koo, T. K., & Li, M. Y. (2016). A guideline of selecting and reporting intraclass correlation coefficients for reliability research. Journal of Chiropractic Medicine, 15(2), 155-163.

# File: utils-calc.R
# Description: Core ANOVA calculation for all ICC models (one-way & two-way)
# Author: [Clare Gao]
# Date: 2026-5-6
# Dependencies: base R (no additional packages required)

#==============================================================================#
# Function 1: icc_calc_anova
# Core ANOVA computation for ICC: unified one-way/two-way calculation
#==============================================================================#
#' Calculate ANOVA for ICC Models
#'
#' @description
#' Unified function to compute mean squares and degrees of freedom for
#' one-way random and two-way random/mixed ANOVA models. Serves as the
#' foundational calculation for all ICC types, eliminating redundant code.
#'
#' @param data_matrix Numeric matrix. Standardized data matrix from \code{icc_preprocess_data}.
#' @param model_type Character. Model type: "oneway" (one-way random) or "twoway" (two-way).
#' @param interaction Logical. Default \code{TRUE}. Whether to include subject×rater interaction
#'   term for two-way models (follows standard literature settings). If FALSE, rater effect
#'   is pooled into the residual term.
#'
#' @return A named list containing ANOVA results:
#' \describe{
#'   \item{MSR}{Numeric. Mean square for subjects (rows).}
#'   \item{MSC}{Numeric. Mean square for raters (columns) (NULL for one-way model).}
#'   \item{MSE}{Numeric. Residual mean square (two-way model only).}
#'   \item{MSW}{Numeric. Within-group mean square (one-way model only).}
#'   \item{df1}{Integer. Degrees of freedom for subjects (n-1).}
#'   \item{df2}{Integer. Residual degrees of freedom.}
#'   \item{df3}{Integer. Degrees of freedom for raters (k-1) (NULL for one-way model).}
#'   \item{n}{Integer. Number of subjects.}
#'   \item{k}{Integer. Number of raters.}
#' }
#'
#' @keywords internal
icc_calc_anova <- function(data_matrix, model_type, interaction = TRUE) {
  # Initialize output list
  result <- list(
    MSR = NULL,
    MSC = NULL,
    MSE = NULL,
    MSW = NULL,
    df1 = NULL,
    df2 = NULL,
    df3 = NULL,
    n = NULL,
    k = NULL
  )

  #----------------------------------------------------------------------------#
  # Step 1: Input validation
  #----------------------------------------------------------------------------#
  # Validate data matrix
  if (!is.matrix(data_matrix) || !is.numeric(data_matrix)) {
    stop("data_matrix must be a numeric matrix (output from icc_preprocess_data)")
  }
  n <- nrow(data_matrix)
  k <- ncol(data_matrix)
  if (n < 2 || k < 2) stop("Data requires at least 2 subjects and 2 raters")

  # Validate model type
  if (!model_type %in% c("oneway", "twoway")) {
    stop("model_type must be 'oneway' or 'twoway'")
  }

  # Store sample sizes
  result$n <- n
  result$k <- k

  #----------------------------------------------------------------------------#
  # Step 2: One-way random ANOVA model (ICC(1,1), ICC(1,k))
  #----------------------------------------------------------------------------#
  if (model_type == "oneway") {
    # Grand mean
    grand_mean <- mean(data_matrix, na.rm = TRUE)

    # Sum of squares
    ss_r <- sum(k * (rowMeans(data_matrix, na.rm = TRUE) - grand_mean)^2)
    ss_w <- sum((data_matrix - rowMeans(data_matrix, na.rm = TRUE))^2)

    # Degrees of freedom
    df1 <- n - 1  # Subjects
    df2 <- n * (k - 1)  # Within groups

    # Mean squares
    ms_r <- ss_r / df1
    ms_w <- ss_w / df2

    # Assign results (one-way: MSC, df3 = NULL)
    result$MSR <- ms_r
    result$MSW <- ms_w
    result$df1 <- df1
    result$df2 <- df2
  }

  #----------------------------------------------------------------------------#
  # Step 3: Two-way ANOVA model (all other ICC types)
  #----------------------------------------------------------------------------#
  if (model_type == "twoway") {
    # Grand mean & marginal means
    grand_mean <- mean(data_matrix, na.rm = TRUE)
    row_means <- rowMeans(data_matrix, na.rm = TRUE)
    col_means <- colMeans(data_matrix, na.rm = TRUE)

    # Sum of squares
    ss_r <- sum(k * (row_means - grand_mean)^2)  # Subjects
    ss_c <- sum(n * (col_means - grand_mean)^2)  # Raters
    ss_e <- sum((data_matrix - row_means - col_means + grand_mean)^2)  # Residual/Interaction

    # Degrees of freedom
    df1 <- n - 1    # Subjects
    df3 <- k - 1    # Raters

    # Handle interaction term
    if (interaction) {
      df2 <- (n - 1) * (k - 1)  # Residual df (with interaction)
    } else {
      # Merge rater effect into residual if interaction = FALSE
      # This is equivalent to assuming no rater effect variance
      ss_e <- ss_e + ss_c
      df2 <- (n - 1) * (k - 1) + (k - 1)
      ss_c <- 0
      df3 <- 0
    }

    # Mean squares
    ms_r <- ss_r / df1
    ms_c <- ifelse(df3 == 0, 0, ss_c / df3)
    ms_e <- ss_e / df2

    # Assign results (two-way: MSW = NULL)
    result$MSR <- ms_r
    result$MSC <- ms_c
    result$MSE <- ms_e
    result$df1 <- df1
    result$df2 <- df2
    result$df3 <- df3
  }

  return(result)
}


#==============================================================================#
# Function 2: icc_tool_point
# Unified point estimate for 10 ICC types (Table 4/5, McGraw & Wong 1996)
#==============================================================================#
#' Calculate ICC Point Estimate
#'
#' @description
#' Unified low-level function to compute point estimates for all 10 ICC types
#' based on the standard formulas from McGraw & Wong (1996) Table 4 and 5.
#' Serves as the single source of truth for ICC point calculations.
#'
#' @param anova_result List. Output from \code{icc_calc_anova} containing ANOVA statistics.
#' @param icc_type Character. ICC type code, must be one of:
#'   "1,1", "1,k", "2,1", "2,k", "3,1", "3,k",
#'   "2,1,consistency", "2,k,consistency", "3,1,absolute", "3,k,absolute"
#'
#' @return Numeric. Point estimate of the intraclass correlation coefficient.
#'
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
#' intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
icc_tool_point <- function(anova_result, icc_type) {
  #----------------------------------------------------------------------------#
  # Step 1: Input validation
  #----------------------------------------------------------------------------#
  # Validate ANOVA result input
  if (!is.list(anova_result)) {
    stop("anova_result must be the output list from icc_calc_anova()")
  }
  required_elements <- c("MSR", "MSW", "MSC", "MSE", "n", "k")
  if (!all(required_elements %in% names(anova_result))) {
    stop("anova_result is missing required ANOVA components")
  }

  # Validate ICC type
  valid_icc_types <- c(
    "1,1", "1,k", "2,1", "2,k", "3,1", "3,k",
    "2,1,consistency", "2,k,consistency",
    "3,1,absolute", "3,k,absolute"
  )
  if (!icc_type %in% valid_icc_types) {
    stop(sprintf("icc_type must be one of: %s", paste(valid_icc_types, collapse = ", ")))
  }

  #----------------------------------------------------------------------------#
  # Step 2: Extract key ANOVA values
  #----------------------------------------------------------------------------#
  MSR <- anova_result$MSR
  MSW <- anova_result$MSW
  MSC <- anova_result$MSC
  MSE <- anova_result$MSE
  n   <- anova_result$n
  k   <- anova_result$k


  # Prevent potential errors when MSC is NULL (e.g., two-way model with interaction = FALSE)
  MSC <- ifelse(is.null(MSC), 0, MSC)

  # For Type C ICC (consistency), MSW may be NULL in two-way model; use MSE as fallback
  if (icc_type %in% c("3,1","3,k","2,1,consistency","2,k,consistency")) {
    if (is.null(MSW)) MSW <- MSE
  }

  #----------------------------------------------------------------------------#
  # Step 3: Calculate point estimate by ICC type (100% match Table 4/5 1996)
  #----------------------------------------------------------------------------#
  icc_point <- switch(
    EXPR = icc_type,
    # One-way models
    "1,1" = (MSR - MSW) / (MSR + (k - 1) * MSW),
    "1,k" = (MSR - MSW) / MSR,

    # Two-way random - absolute agreement
    "2,1" = (MSR - MSE) / (MSR + (k - 1) * MSE + k * (MSC - MSE) / n),
    "2,k" = (MSR - MSE) / (MSR + (MSC - MSE) / n),

    # Two-way mixed - consistency
    "3,1" = (MSR - MSE) / (MSR + (k - 1) * MSE),
    "3,k" = (MSR - MSE) / MSR,

    # Random + consistency (mathematically equivalent to 3,1 / 3,k)
    "2,1,consistency" = (MSR - MSE) / (MSR + (k - 1) * MSE),
    "2,k,consistency" = (MSR - MSE) / MSR,

    # Mixed + absolute agreement (mathematically equivalent to 2,1 / 2,k)
    "3,1,absolute" = (MSR - MSE) / (MSR + (k - 1) * MSE + k * (MSC - MSE) / n),
    "3,k,absolute" = (MSR - MSE) / (MSR + (MSC - MSE) / n)
  )

  #----------------------------------------------------------------------------#
  # Step 4: Return final point estimate
  #----------------------------------------------------------------------------#
  return(icc_point)
}

#==============================================================================#
# Function 3: icc_tool_ci
# Exact F-distribution confidence intervals for ICC (with Satterthwaite correction)
#==============================================================================#
#' Calculate ICC Confidence Intervals
#'
#' @description
#' Computes confidence intervals for all 10 ICC types using exact F-distribution
#' method from McGraw & Wong (1996) Table 7. Implements Satterthwaite degrees of
#' freedom correction for Type A ICC models (ICC(2,1), ICC(2,k) and their variants).
#'
#' @param anova_result List. Output from \code{icc_calc_anova}.
#' @param icc_type Character. ICC type code (matches icc_tool_point).
#' @param point_est Numeric. Point estimate from \code{icc_tool_point}.
#' @param alpha Numeric. Significance level, default 0.05 (95\% CI).
#'
#' @return A named list with CI results:
#' \describe{
#'   \item{ci_level}{Numeric. Confidence level (1 - alpha).}
#'   \item{ci_lower}{Numeric. Lower bound of CI (truncated to 0).}
#'   \item{ci_upper}{Numeric. Upper bound of CI (truncated to 1).}
#'   \item{df_corrected}{Numeric or NULL. Satterthwaite-corrected df (Type A only).}
#' }
#'
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
#' intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
icc_tool_ci <- function(anova_result, icc_type, point_est, alpha = 0.05) {
  # Initialize output
  result <- list(
    ci_level = 1 - alpha,
    ci_lower = NULL,
    ci_upper = NULL,
    df_corrected = NULL
  )

  # Input validation
  valid_icc <- c("1,1","1,k","2,1","2,k","3,1","3,k","2,1,consistency","2,k,consistency","3,1,absolute","3,k,absolute")
  if (!icc_type %in% valid_icc) stop("Invalid icc_type")
  if (!is.list(anova_result)) stop("Invalid anova_result")

  # Extract parameters
  MSR <- anova_result$MSR
  MSW <- anova_result$MSW
  MSC <- anova_result$MSC
  MSE <- anova_result$MSE
  n <- anova_result$n
  k <- anova_result$k
  df1 <- anova_result$df1
  df2 <- anova_result$df2
  df3 <- anova_result$df3

  # Prevent potential errors when MSC is NULL (e.g., two-way model with interaction = FALSE)
  MSC <- ifelse(is.null(MSC), 0, MSC)

  # For Type C ICC (consistency), MSW may be NULL in two-way model; use MSE as fallback
  if (icc_type %in% c("3,1","3,k","2,1,consistency","2,k,consistency")) {
    if (is.null(MSW)) MSW <- MSE
  }

  # F quantiles (two-tailed)
  F_lower <- stats::qf(1 - alpha/2, df1, df2)  # 上界对应右尾
  F_upper <- stats::qf(alpha/2, df1, df2)

  # CI calculation by ICC type
  if (icc_type %in% c("1,1")) {
    F1 <- MSR / MSW
    lower <- (F1 / F_lower - 1) / (F1 / F_lower + (k-1))
    upper <- (F1 / F_upper - 1) / (F1 / F_upper + (k-1))
  } else if (icc_type %in% c("1,k")) {
    F1 <- MSR / MSW
    lower <- 1 - 1/F_lower
    upper <- 1 - 1/F_upper
  } else if (icc_type %in% c("3,1", "2,1,consistency")) {
    F1 <- MSR / MSE
    lower <- (F1 / F_lower - 1) / (F1 / F_lower + (k-1))
    upper <- (F1 / F_upper - 1) / (F1 / F_upper + (k-1))
  } else if (icc_type %in% c("3,k", "2,k,consistency")) {
    F1 <- MSR / MSE
    lower <- 1 - 1/F_lower
    upper <- 1 - 1/F_upper
  } else if (icc_type %in% c("2,1", "3,1,absolute", "2,k", "3,k,absolute")) {
    # Type A: Satterthwaite degrees of freedom correction (McGraw & Wong 1996, p.37)
    a <- k * MSC / n + (n - 1) * MSE
    b <- (n - 1) * k * MSE / n
    df_corrected <- (a + b)^2 / (a^2 / df1 + b^2 / df2)
    result$df_corrected <- df_corrected

    # Recalculate F quantiles with corrected degrees of freedom
    F_lower_corr <- stats::qf(1 - alpha/2, df1, df_corrected)  # 右尾
    F_upper_corr <- stats::qf(alpha/2, df1, df_corrected)

    if (icc_type %in% c("2,1", "3,1,absolute")) {
      F_val <- MSR / MSE
      # Modified Type A ICC CI formula 5.16
      lower <- (F_val / F_lower_corr - 1) / (F_val / F_lower_corr + (k-1) + k*(MSC - MSE)/(n*MSE))
      upper <- (F_val / F_upper_corr - 1) / (F_val / F_upper_corr + (k-1) + k*(MSC - MSE)/(n*MSE))
    } else {
      F_val <- MSR / MSE
      lower <- 1 - ( (MSR + (MSC-MSE)/n ) / (MSE * F_lower_corr) ) / (MSR/MSE)
      upper <- 1 - ( (MSR + (MSC-MSE)/n ) / (MSE * F_upper_corr) ) / (MSR/MSE)
    }
  }

  # Truncate CI to valid ICC range [0, 1]
  result$ci_lower <- max(lower, 0)
  result$ci_upper <- min(upper, 1)

  return(result)
}

#==============================================================================#
# Function 4: icc_calc_f_test
# Unified F-test for ICC (H0: ICC = 0 or non-zero rho0)
#==============================================================================#
#' Hypothesis Testing for ICC
#'
#' @description
#' Performs one-tailed F-test for ICC significance following McGraw & Wong (1996) Table 8.
#' Supports both null hypothesis of zero ICC and custom non-zero null value.
#'
#' @param anova_result List. Output from \code{icc_calc_anova}.
#' @param icc_type Character. ICC type code.
#' @param rho0 Numeric. Null hypothesis ICC value, default 0.
#' @param alpha Numeric. Significance level, default 0.05.
#'
#' @return Named list with test results:
#' \describe{
#'   \item{H0}{Character. Null hypothesis statement.}
#'   \item{F_stat}{Numeric. F test statistic.}
#'   \item{df1}{Integer. Numerator degrees of freedom.}
#'   \item{df2}{Integer. Denominator degrees of freedom.}
#'   \item{p_value}{Numeric. One-tailed p-value (H1: ICC > rho0).}
#' }
#'
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
#' intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
icc_calc_f_test <- function(anova_result, icc_type, rho0 = 0, alpha = 0.05) {
  # Initialize output
  result <- list(
    H0 = paste0("ICC = ", rho0),
    F_stat = NULL,
    df1 = NULL,
    df2 = NULL,
    p_value = NULL
  )

  # Input validation
  if (!is.list(anova_result)) stop("Invalid anova_result")
  if (rho0 < 0 || rho0 > 1) stop("rho0 must be between 0 and 1")

  # Extract parameters
  MSR <- anova_result$MSR
  MSW <- anova_result$MSW
  MSC <- anova_result$MSC
  MSE <- anova_result$MSE
  n <- anova_result$n
  k <- anova_result$k
  df1 <- anova_result$df1
  df2 <- anova_result$df2

  # F statistic calculation
  if (rho0 == 0) {
    # Standard zero test (McGraw & Wong 1996, Table 8)
    if (icc_type %in% c("1,1", "1,k")) {
      result$F_stat <- MSR / MSW
      result$df1 <- df1
      result$df2 <- n*(k-1)
    } else if (icc_type %in% c("3,1","3,k","2,1,consistency","2,k,consistency")) {
      result$F_stat <- MSR / MSE
      result$df1 <- df1
      result$df2 <- df2
    } else if (icc_type %in% c("2,1","2,k","3,1,absolute","3,k,absolute")) {
      result$F_stat <- MSR / MSE
      result$df1 <- df1
      result$df2 <- df2
    }
  } else {
    # Non-zero test (McGraw & Wong 1996, Table 8)
    if (icc_type %in% c("1,1", "1,k")) {
      result$F_stat <- (MSR/MSW) * (1 - rho0) / (1 + (k-1)*rho0)
      result$df1 <- df1
      result$df2 <- n*(k-1)
    } else if (icc_type %in% c("3,1","3,k","2,1,consistency","2,k,consistency")) {
      result$F_stat <- (MSR/MSE) * (1 - rho0) / (1 + (k-1)*rho0)
      result$df1 <- df1
      result$df2 <- df2
    } else if (icc_type %in% c("2,1","2,k","3,1,absolute","3,k,absolute")) {
      # Type A models: include rater variance component
      numerator <- MSR * (1 - rho0)
      denominator <- MSE * (1 + (k-1)*rho0) + k * (MSC - MSE) * rho0 / n
      result$F_stat <- numerator / denominator
      result$df1 <- df1
      result$df2 <- df2
    }
  }

  # One-tailed p-value (standard for reliability: H1: ICC > rho0)
  result$p_value <- stats::pf(result$F_stat, result$df1, result$df2, lower.tail = FALSE)

  return(result)
}

# File: sample_size_calculate.R
# Description: Sample size and power analysis for ICC
# Author: [Ziyu Liu]
# Date: [20260421]

#' Sample Size for ICC based on Lower Confidence Limit
#'
#' @param rho Anticipated ICC value.
#' @param rho0 Desired lower bound.
#' @param k Number of observations per subject. Default 3.
#' @param same_raters Logical.
#' @param rater_effect "random" or "fixed".
#' @param rating_type "single" or "average".
#' @param agreement_type "absolute" or "consistency".
#' @param alpha Significance level. Default 0.05.
#' @param assurance Assurance probability. Default 0.8.
#' @param rating_target Shortcut for rho0.
#' @param verbose Print messages. Default TRUE.
#'
#' @return Required sample size.
#' @export
#'
#' @examples
#' # Note: It is recommended to use the unified interface icc_sample_size()
#' icc_sample_size_lower(rho = 0.8, rho0 = 0.6, k = 3, same_raters = FALSE, rating_type = "single")

icc_sample_size_lower <- function(
    rho, rho0 = NULL, k = 3, same_raters, rater_effect = NULL,
    rating_type, agreement_type = NULL, alpha = 0.05, assurance = 0.8,
    rating_target = NULL, verbose = TRUE
) {
  if (!is.null(rating_target)) {
    rating_map <- c(poor=0.5, moderate=0.5, good=0.75, excellent=0.9)
    if (!rating_target %in% names(rating_map)) stop("rating_target error")
    rho0 <- rating_map[rating_target]
  }
  if (is.null(rho0)) stop("Provide rho0 or rating_target")

  design_check <- icc_check_design(same_raters, rater_effect, rating_type, agreement_type, k)
  if (!design_check$is_valid) stop(design_check$error_msg)
  mapping <- icc_map_design_to_icc(same_raters, rater_effect, rating_type, agreement_type)

  if (verbose) message("Mapped ICC type: ", mapping$icc_full_name)
  stopifnot(rho>=0&rho<=1, rho0>=0&rho0<=1, rho>rho0, k>=2)

  z_alpha <- stats::qnorm(1-alpha)
  z_beta <- stats::qnorm(assurance)
  F_rho <- (1+(k-1)*rho)/(1-rho)
  F_rho0 <- (1+(k-1)*rho0)/(1-rho0)
  N <- 1 + 2*(z_alpha+z_beta)^2*k / ((log(F_rho/F_rho0))^2*(k-1))
  ceiling(N)
}

#' Sample Size for ICC based on Confidence Interval Width
#'
#' @param rho Anticipated ICC value.
#' @param omega Desired half-width.
#' @param k Number of observations. Default 3.
#' @param same_raters Logical.
#' @param rater_effect "random" or "fixed".
#' @param rating_type "single" or "average".
#' @param agreement_type "absolute" or "consistency".
#' @param alpha Significance level. Default 0.05.
#' @param assurance Assurance probability. Default 0.8.
#' @param verbose Print messages. Default TRUE.
#'
#' @return Required sample size.
#' @export
#'
#' @examples
#' # Note: It is recommended to use the unified interface icc_sample_size()
#' icc_sample_size_width(rho = 0.7, omega = 0.1, k = 3, same_raters = FALSE, rating_type = "average")

icc_sample_size_width <- function(
    rho, omega, k=3, same_raters, rater_effect=NULL, rating_type,
    agreement_type=NULL, alpha=0.05, assurance=0.8, verbose=TRUE
) {
  design_check <- icc_check_design(same_raters, rater_effect, rating_type, agreement_type, k)
  if (!design_check$is_valid) stop(design_check$error_msg)
  if (verbose) message("Mapped ICC type.")
  stopifnot(rho>=0&rho<=1, omega>0, k>=2)

  z_alpha2 <- stats::qnorm(1-alpha/2)
  z_beta <- stats::qnorm(assurance)
  A <- (1-rho)*(1+(k-1)*rho)
  B <- k-2+2*rho-2*k*rho
  absB <- abs(B)

  term1 <- A*z_alpha2
  term2 <- sqrt(A^2*z_alpha2^2 + 4*omega*z_alpha2*z_beta*A*absB)
  sqrt_N_minus_1 <- (term1+term2)/(omega*sqrt(2*k*(k-1)))
  N <- 1 + sqrt_N_minus_1^2
  ceiling(N)
}

#' Power Calculation for ICC Study Design
#'
#' @param n Number of subjects.
#' @param rho Anticipated ICC value.
#' @param rho0 Lower bound for method="lower".
#' @param omega Half-width for method="width".
#' @param k Number of observations. Default 3.
#' @param same_raters Logical.
#' @param rater_effect "random" or "fixed".
#' @param rating_type "single" or "average".
#' @param agreement_type "absolute" or "consistency".
#' @param alpha Significance level. Default 0.05.
#' @param method "lower" or "width".
#' @param verbose Print messages. Default TRUE.
#'
#' @return Power.
#' @export
#'
#' @examples
#' # Note: It is recommended to use the unified interface icc_sample_size()
#' icc_power(n = 30, rho = 0.7, rho0 = 0.5, k = 3,
#'           same_raters = TRUE, rater_effect = "fixed",
#'           rating_type = "single", agreement_type = "consistency")

icc_power <- function(
    n, rho, rho0=NULL, omega=NULL, k=3, same_raters, rater_effect=NULL,
    rating_type, agreement_type=NULL, alpha=0.05, method=c("lower","width"), verbose=TRUE
) {
  method <- match.arg(method)
  stopifnot(n>=2, rho>=0&rho<=1)

  if (method=="lower") {
    if (is.null(rho0)) stop("Provide rho0")
    z_alpha <- stats::qnorm(1-alpha)
    F_rho <- (1+(k-1)*rho)/(1-rho)
    F_rho0 <- (1+(k-1)*rho0)/(1-rho0)
    z_beta <- log(F_rho/F_rho0)*sqrt((k-1)*(n-1)/(2*k)) - z_alpha
    return(stats::pnorm(z_beta))
  }

  if (method=="width") {
    if (is.null(omega)) stop("Provide omega")
    z_alpha2 <- stats::qnorm(1-alpha/2)
    A <- (1-rho)*(1+(k-1)*rho)
    B <- k-2+2*rho-2*k*rho
    absB <- abs(B)
    numerator <- omega*sqrt(2*k*(k-1))*sqrt(n-1) - A*z_alpha2
    denominator <- 2*omega*z_alpha2*A*absB
    z_beta <- sqrt(numerator^2/denominator)
    return(stats::pnorm(z_beta))
  }
}

#' Unified ICC Sample Size & Power Interface
#'
#' @param method "lower", "width", "power".
#' @param ... Arguments passed to underlying functions.
#'
#' @return Sample size or power.
#' @export
#'
#' @examples
#' # Method 1: Sample size based on lower confidence limit (most recommended)
#' # Ensure 95% CI lower bound >= 0.75 (good reliability)
#' n1 <- icc_sample_size(
#'   method = "lower",
#'   rho = 0.85,
#'   rating_target = "good",
#'   k = 3,
#'   same_raters = TRUE,
#'   rater_effect = "random",
#'   rating_type = "single",
#'   agreement_type = "absolute"
#' )
#'
#' # Method 2: Sample size based on confidence interval width
#' # Ensure 95% CI half-width <= 0.1
#' n2 <- icc_sample_size(
#'   method = "width",
#'   rho = 0.7,
#'   omega = 0.1,
#'   k = 3,
#'   same_raters = FALSE,
#'   rating_type = "average"
#' )
#'
#' # Method 3: Power calculation for existing study design
#' power <- icc_sample_size(
#'   method = "power",
#'   n = 30,
#'   rho = 0.7,
#'   rho0 = 0.5,
#'   k = 3,
#'   same_raters = TRUE,
#'   rater_effect = "fixed",
#'   rating_type = "single",
#'   agreement_type = "consistency"
#' )
icc_sample_size <- function(method = c("lower", "width", "power"), ...) {
  method <- match.arg(method)
  switch(method,
         lower = icc_sample_size_lower(...),
         width = icc_sample_size_width(...),
         power = icc_power(...)
  )
}

# File: shiny-app.R
# Description: Interactive Shiny application for ICCDesign

#' Launch the ICCDesign Shiny Application
#'
#' @description
#' Starts an interactive Shiny application for ICC analysis, reliability
#' reporting, and sample size or power planning.
#'
#' @param host Host address passed to \code{shiny::runApp}. Default is
#'   \code{"127.0.0.1"}.
#' @param port Optional port passed to \code{shiny::runApp}. If \code{NULL},
#'   Shiny selects an available port.
#' @param launch.browser Logical. Whether to open the app in a browser.
#'   Default is \code{interactive()}.
#' @param ... Additional arguments passed to \code{shiny::runApp}.
#'
#' @return Runs the Shiny application.
#' @export
#'
#' @examples
#' if (interactive()) {
#' # Launch the interactive Shiny application
#' run_icc_app()
#' }
run_icc_app <- function(host = "127.0.0.1", port = NULL,
                        launch.browser = interactive(), ...) {
  shiny::runApp(
    appDir = icc_shiny_app(),
    host = host,
    port = port,
    launch.browser = launch.browser,
    ...
  )
}

#' Build the ICCDesign Shiny Application
#'
#' @return A Shiny application object.
#' @keywords internal
icc_shiny_app <- function() {
  shiny::shinyApp(ui = .icc_app_ui(), server = .icc_app_server)
}

.icc_app_ui <- function() {
  shiny::fluidPage(
    shiny::tags$head(
      shiny::tags$meta(name = "viewport", content = "width=device-width, initial-scale=1"),
      shiny::tags$style(shiny::HTML(.icc_app_css()))
    ),
    shiny::div(
      class = "app-shell",
      shiny::div(
        class = "topbar",
        shiny::div(
          class = "brand-block",
          shiny::div(class = "brand-mark", "ICC"),
          shiny::div(
            shiny::h1("ICCDesign"),
            shiny::p("Interactive reliability analysis and study planning")
          )
        ),
        shiny::div(
          class = "topbar-actions",
          shiny::span(class = "status-pill", "Point estimate"),
          shiny::span(class = "status-pill", "Exact CI"),
          shiny::span(class = "status-pill", "Power")
        )
      ),
      shiny::tabsetPanel(
        id = "main_tabs",
        type = "tabs",
        shiny::tabPanel(
          title = shiny::tagList(shiny::icon("calculator"), "ICC analysis"),
          shiny::div(
            class = "workspace-grid",
            .icc_analysis_controls(),
            shiny::div(
              class = "result-stack",
              shiny::uiOutput("analysis_alert"),
              shiny::uiOutput("analysis_metrics"),
              shiny::div(
                class = "two-col",
                shiny::div(
                  class = "panel",
                  shiny::h3("Reliability interval"),
                  shiny::plotOutput("icc_plot", height = "230px")
                ),
                shiny::div(
                  class = "panel",
                  shiny::h3("Model summary"),
                  shiny::tableOutput("model_summary")
                )
              ),
              shiny::div(
                class = "two-col",
                shiny::div(
                  class = "panel",
                  shiny::h3("ANOVA components"),
                  shiny::tableOutput("anova_table")
                ),
                shiny::div(
                  class = "panel",
                  shiny::h3("Hypothesis tests"),
                  shiny::tableOutput("f_test_table")
                )
              ),
              shiny::div(
                class = "panel",
                shiny::div(
                  class = "panel-heading",
                  shiny::h3("Publication-ready report"),
                  shiny::downloadButton(
                    "download_report",
                    label = "Download"
                  )
                ),
                shiny::verbatimTextOutput("report_text")
              )
            )
          )
        ),
        shiny::tabPanel(
          title = shiny::tagList(shiny::icon("chart-line"), "Sample size"),
          shiny::div(
            class = "workspace-grid",
            .icc_sample_size_controls(),
            shiny::div(
              class = "result-stack",
              shiny::uiOutput("ss_alert"),
              shiny::uiOutput("ss_metrics"),
              shiny::div(
                class = "two-col",
                shiny::div(
                  class = "panel",
                  shiny::h3("Power curve"),
                  shiny::plotOutput("power_plot", height = "260px")
                ),
                shiny::div(
                  class = "panel",
                  shiny::h3("Planning details"),
                  shiny::tableOutput("ss_details")
                )
              ),
              shiny::div(
                class = "panel",
                shiny::div(
                  class = "panel-heading",
                  shiny::h3("Design export"),
                  shiny::downloadButton(
                    "download_design",
                    label = "Download CSV"
                  )
                ),
                shiny::tableOutput("ss_export_preview")
              )
            )
          )
        ),
        shiny::tabPanel(
          title = shiny::tagList(shiny::icon("table"), "Data and guide"),
          shiny::div(
            class = "guide-grid",
            shiny::div(
              class = "panel",
              shiny::h3("Current data preview"),
              shiny::tableOutput("data_preview")
            ),
            shiny::div(
              class = "panel",
              shiny::h3("Data diagnostics"),
              shiny::tableOutput("data_diagnostics")
            ),
            shiny::div(
              class = "panel",
              shiny::h3("ICC selection guide"),
              shiny::tableOutput("selection_guide")
            ),
            shiny::div(
              class = "panel",
              shiny::h3("Reliability thresholds"),
              shiny::tableOutput("rating_guide")
            )
          )
        )
      )
    )
  )
}

.icc_analysis_controls <- function() {
  shiny::div(
    class = "control-panel",
    shiny::h2("Analysis setup"),
    shiny::p(class = "muted", "Upload or paste a numeric matrix, then choose the design."),
    shiny::radioButtons(
      "data_mode",
      "Data source",
      choices = c("Example data" = "example", "Upload file" = "upload", "Paste table" = "paste"),
      selected = "example",
      inline = TRUE
    ),
    shiny::conditionalPanel(
      "input.data_mode == 'example'",
      shiny::div(
        class = "example-note",
        shiny::p(class = "muted", "Uses the fixed built-in icc_data dataset.")
      )
    ),
    shiny::conditionalPanel(
      "input.data_mode == 'upload'",
      shiny::fileInput("data_file", "CSV, TSV, or TXT file", accept = c(".csv", ".tsv", ".txt")),
      shiny::div(
        class = "input-grid",
        shiny::checkboxInput("file_header", "First row contains column names", value = TRUE),
        shiny::selectInput("file_sep", "Separator", choices = .icc_sep_choices(), selected = ","),
        shiny::selectInput("file_dec", "Decimal mark", choices = c("." = ".", "," = ","), selected = ".")
      )
    ),
    shiny::conditionalPanel(
      "input.data_mode == 'paste'",
      shiny::textAreaInput(
        "pasted_data",
        "Paste numeric table",
        value = paste(
          "Rater1,Rater2,Rater3",
          "76,79,77",
          "83,82,85",
          "91,89,90",
          "68,70,67",
          "74,75,76",
          sep = "\n"
        ),
        rows = 7
      ),
      shiny::div(
        class = "input-grid",
        shiny::checkboxInput("pasted_header", "First row contains column names", value = TRUE),
        shiny::selectInput("pasted_sep", "Separator", choices = .icc_sep_choices(), selected = ","),
        shiny::selectInput("pasted_dec", "Decimal mark", choices = c("." = ".", "," = ","), selected = ".")
      )
    ),
    shiny::hr(),
    shiny::h2("Design questions"),
    shiny::radioButtons(
      "same_raters",
      "Do all subjects share the same raters?",
      choices = c("Yes" = "TRUE", "No" = "FALSE"),
      selected = "TRUE",
      inline = TRUE
    ),
    shiny::conditionalPanel(
      "input.same_raters == 'TRUE'",
      shiny::div(
        class = "input-grid",
        shiny::selectInput("rater_effect", "Rater effect", choices = c("Random" = "random", "Fixed" = "fixed")),
        shiny::selectInput(
          "agreement_type",
          "Agreement target",
          choices = c("Absolute agreement" = "absolute", "Consistency" = "consistency")
        )
      )
    ),
    shiny::selectInput("rating_type", "Rating unit", choices = c("Single rating" = "single", "Average rating" = "average")),
    shiny::div(
      class = "input-grid",
      shiny::numericInput("alpha", "Alpha", value = 0.05, min = 0.001, max = 0.5, step = 0.005),
      shiny::checkboxInput("na_rm", "Remove rows with missing values", value = TRUE),
      shiny::checkboxInput("interaction", "Include subject-rater interaction", value = TRUE),
      shiny::checkboxInput("use_rho0", "Test against custom rho0", value = FALSE)
    ),
    shiny::conditionalPanel(
      "input.use_rho0 == true",
      shiny::numericInput("rho0", "Null ICC rho0", value = 0.5, min = 0, max = 0.99, step = 0.01)
    ),
    shiny::actionButton(
      "run_analysis",
      label = shiny::tagList(shiny::icon("play"), "Run analysis"),
      class = "primary-action"
    )
  )
}

.icc_sample_size_controls <- function() {
  shiny::div(
    class = "control-panel",
    shiny::h2("Study planning"),
    shiny::p(class = "muted", "Estimate required subjects or the assurance probability for a fixed sample size."),
    shiny::radioButtons(
      "ss_method",
      "Planning method",
      choices = c(
        "Lower CI assurance" = "lower",
        "CI half-width assurance" = "width",
        "Power for fixed n" = "power"
      ),
      selected = "lower"
    ),
    shiny::div(
      class = "input-grid",
      shiny::numericInput("ss_rho", "Anticipated ICC", value = 0.75, min = 0.01, max = 0.99, step = 0.01),
      shiny::numericInput("ss_k", "Raters per subject", value = 3, min = 2, max = 50, step = 1),
      shiny::numericInput("ss_alpha", "Alpha", value = 0.05, min = 0.001, max = 0.5, step = 0.005),
      shiny::numericInput("ss_assurance", "Assurance target", value = 0.8, min = 0.01, max = 0.99, step = 0.01)
    ),
    shiny::conditionalPanel(
      "input.ss_method == 'lower'",
      shiny::numericInput("ss_lower_rho0", "Required lower confidence limit", value = 0.5, min = 0, max = 0.98, step = 0.01)
    ),
    shiny::conditionalPanel(
      "input.ss_method == 'width'",
      shiny::numericInput("ss_width_omega", "Target CI half-width", value = 0.1, min = 0.001, max = 0.8, step = 0.01)
    ),
    shiny::conditionalPanel(
      "input.ss_method == 'power'",
      shiny::div(
        class = "input-grid",
        shiny::numericInput("ss_n", "Subjects", value = 40, min = 2, max = 10000, step = 1),
        shiny::radioButtons(
          "ss_power_method",
          "Power target",
          choices = c("Lower CI" = "lower", "CI width" = "width"),
          selected = "lower",
          inline = TRUE
        )
      ),
      shiny::conditionalPanel(
        "input.ss_power_method == 'lower'",
        shiny::numericInput("ss_power_rho0", "Lower CI threshold", value = 0.5, min = 0, max = 0.98, step = 0.01)
      ),
      shiny::conditionalPanel(
        "input.ss_power_method == 'width'",
        shiny::numericInput("ss_power_omega", "CI half-width", value = 0.1, min = 0.001, max = 0.8, step = 0.01)
      )
    ),
    shiny::hr(),
    shiny::h2("Design questions"),
    shiny::radioButtons(
      "ss_same_raters",
      "Do all subjects share the same raters?",
      choices = c("Yes" = "TRUE", "No" = "FALSE"),
      selected = "TRUE",
      inline = TRUE
    ),
    shiny::conditionalPanel(
      "input.ss_same_raters == 'TRUE'",
      shiny::div(
        class = "input-grid",
        shiny::selectInput("ss_rater_effect", "Rater effect", choices = c("Random" = "random", "Fixed" = "fixed")),
        shiny::selectInput(
          "ss_agreement_type",
          "Agreement target",
          choices = c("Absolute agreement" = "absolute", "Consistency" = "consistency")
        )
      )
    ),
    shiny::selectInput("ss_rating_type", "Rating unit", choices = c("Single rating" = "single", "Average rating" = "average")),
    shiny::actionButton(
      "run_ss",
      label = shiny::tagList(shiny::icon("play"), "Calculate"),
      class = "primary-action"
    )
  )
}

.icc_app_server <- function(input, output, session) {
  raw_data <- shiny::reactive({
    switch(
      input$data_mode,
      example = .icc_example_data(),
      upload = .icc_read_uploaded(input$data_file, input$file_header, input$file_sep, input$file_dec),
      paste = .icc_read_pasted(input$pasted_data, input$pasted_header, input$pasted_sep, input$pasted_dec),
      NULL
    )
  })

  analysis <- shiny::eventReactive(input$run_analysis, {
    tryCatch({
      data <- raw_data()
      if (is.null(data)) {
        stop("Provide numeric data before running the analysis.", call. = FALSE)
      }

      design <- .icc_design_args(input)
      result <- do.call(
        icc_calc,
        c(
          list(
            data = data,
            alpha = input$alpha,
            rho0 = if (isTRUE(input$use_rho0)) input$rho0 else NULL,
            interaction = isTRUE(input$interaction),
            na.rm = isTRUE(input$na_rm),
            verbose = FALSE
          ),
          design
        )
      )

      list(ok = TRUE, value = result)
    }, error = function(e) {
      list(ok = FALSE, error = conditionMessage(e))
    })
  }, ignoreNULL = FALSE)

  ss_result <- shiny::eventReactive(input$run_ss, {
    tryCatch({
      design <- .icc_design_args(input, prefix = "ss")
      method <- input$ss_method

      value <- switch(
        method,
        lower = do.call(
          icc_sample_size_lower,
          c(
            list(
              rho = input$ss_rho,
              rho0 = input$ss_lower_rho0,
              k = input$ss_k,
              alpha = input$ss_alpha,
              assurance = input$ss_assurance,
              verbose = FALSE
            ),
            design
          )
        ),
        width = do.call(
          icc_sample_size_width,
          c(
            list(
              rho = input$ss_rho,
              omega = input$ss_width_omega,
              k = input$ss_k,
              alpha = input$ss_alpha,
              assurance = input$ss_assurance,
              verbose = FALSE
            ),
            design
          )
        ),
        power = do.call(
          icc_power,
          c(
            list(
              n = input$ss_n,
              rho = input$ss_rho,
              rho0 = if (input$ss_power_method == "lower") input$ss_power_rho0 else NULL,
              omega = if (input$ss_power_method == "width") input$ss_power_omega else NULL,
              k = input$ss_k,
              alpha = input$ss_alpha,
              method = input$ss_power_method,
              verbose = FALSE
            ),
            design
          )
        )
      )

      mapping <- do.call(icc_map_design_to_icc, design)
      list(ok = TRUE, value = value, method = method, design = design, mapping = mapping)
    }, error = function(e) {
      list(ok = FALSE, error = conditionMessage(e))
    })
  }, ignoreNULL = FALSE)

  output$analysis_alert <- shiny::renderUI({
    x <- analysis()
    if (!isTRUE(x$ok)) {
      return(.icc_alert("Analysis cannot run", x$error, type = "error"))
    }

    result <- x$value
    messages <- c(
      result$data_summary$warning_msg,
      result$warning_msg,
      result$tip_msg
    )
    messages <- messages[!vapply(messages, is.null, logical(1))]
    if (!length(messages)) {
      return(.icc_alert("Ready", "The selected design and data passed validation.", type = "success"))
    }
    .icc_alert("Review notes", paste(messages, collapse = " "), type = "warning")
  })

  output$analysis_metrics <- shiny::renderUI({
    x <- analysis()
    if (!isTRUE(x$ok)) {
      return(.icc_empty_state("Run a valid analysis to see ICC results."))
    }
    result <- x$value$icc_result
    eval <- x$value$evaluation
    f_test <- result$F_test_null

    shiny::div(
      class = "metric-grid",
      .icc_metric_card("ICC", .icc_fmt(result$point_est, 4), result$icc_code, "accent"),
      .icc_metric_card(
        "Confidence interval",
        paste0("[", .icc_fmt(result$ci_lower, 4), ", ", .icc_fmt(result$ci_upper, 4), "]"),
        paste0(.icc_fmt(result$ci_level * 100, 0), "% CI"),
        "teal"
      ),
      .icc_metric_card("Reliability", eval$rating_en, "Koo and Li lower-bound rule", "green"),
      .icc_metric_card("F-test p-value", .icc_p_value(f_test$p_value), "H0: ICC = 0", "amber")
    )
  })

  output$icc_plot <- shiny::renderPlot({
    x <- analysis()
    shiny::req(isTRUE(x$ok))
    result <- x$value$icc_result
    .icc_plot_interval(result$point_est, result$ci_lower, result$ci_upper)
  })

  output$model_summary <- shiny::renderTable({
    x <- analysis()
    shiny::req(isTRUE(x$ok))
    result <- x$value
    data.frame(
      Item = c("ICC type", "ICC code", "Subjects", "Raters", "Rating", "Reliability"),
      Value = c(
        result$icc_result$icc_type,
        result$icc_result$icc_code,
        result$data_summary$n,
        result$data_summary$k,
        result$evaluation$rating_en,
        result$evaluation$explanation
      ),
      check.names = FALSE
    )
  }, striped = TRUE, bordered = FALSE, spacing = "s")

  output$anova_table <- shiny::renderTable({
    x <- analysis()
    shiny::req(isTRUE(x$ok))
    .icc_named_list_table(x$value$icc_result$anova_summary)
  }, striped = TRUE, bordered = FALSE, spacing = "s")

  output$f_test_table <- shiny::renderTable({
    x <- analysis()
    shiny::req(isTRUE(x$ok))
    .icc_f_test_table(x$value$icc_result)
  }, striped = TRUE, bordered = FALSE, spacing = "s")

  output$report_text <- shiny::renderText({
    x <- analysis()
    if (!isTRUE(x$ok)) {
      return(x$error)
    }
    x$value$report
  })

  output$download_report <- shiny::downloadHandler(
    filename = function() paste0("icc-report-", Sys.Date(), ".txt"),
    content = function(file) {
      x <- analysis()
      text <- if (isTRUE(x$ok)) x$value$report else x$error
      writeLines(text, con = file, useBytes = TRUE)
    }
  )

  output$data_preview <- shiny::renderTable({
    data <- raw_data()
    if (is.null(data)) {
      return(data.frame(Message = "No data loaded.", check.names = FALSE))
    }
    utils::head(as.data.frame(data), 12)
  }, striped = TRUE, bordered = FALSE, spacing = "s", rownames = TRUE)

  output$data_diagnostics <- shiny::renderTable({
    data <- raw_data()
    if (is.null(data)) {
      return(data.frame(Item = "Status", Value = "No data loaded.", check.names = FALSE))
    }
    prep <- icc_preprocess_data(data, na.rm = TRUE)
    data.frame(
      Item = c("Rows", "Columns", "Missing cells", "Validation", "Message"),
      Value = c(
        nrow(as.matrix(data)),
        ncol(as.matrix(data)),
        sum(is.na(as.matrix(data))),
        if (is.null(prep$error_msg)) "Valid numeric matrix" else "Invalid",
        if (is.null(prep$error_msg)) .icc_or(prep$warning_msg, "No issues detected.") else prep$error_msg
      ),
      check.names = FALSE
    )
  }, striped = TRUE, bordered = FALSE, spacing = "s")

  output$selection_guide <- shiny::renderTable({
    .icc_selection_guide()
  }, striped = TRUE, bordered = FALSE, spacing = "s")

  output$rating_guide <- shiny::renderTable({
    data.frame(
      Rating = c("Poor", "Moderate", "Good", "Excellent"),
      Rule = c("Lower CI < 0.50", "0.50 <= lower CI < 0.75", "0.75 <= lower CI < 0.90", "Lower CI >= 0.90"),
      check.names = FALSE
    )
  }, striped = TRUE, bordered = FALSE, spacing = "s")

  output$ss_alert <- shiny::renderUI({
    x <- ss_result()
    if (!isTRUE(x$ok)) {
      return(.icc_alert("Planning cannot run", x$error, type = "error"))
    }
    message <- paste("Mapped design:", x$mapping$icc_full_name)
    extra <- c(x$mapping$warning_msg, x$mapping$tip_msg)
    extra <- extra[!vapply(extra, is.null, logical(1))]
    if (length(extra)) {
      message <- paste(message, paste(extra, collapse = " "))
    }
    .icc_alert("Planning ready", message, type = "success")
  })

  output$ss_metrics <- shiny::renderUI({
    x <- ss_result()
    if (!isTRUE(x$ok)) {
      return(.icc_empty_state("Set a valid planning design to see results."))
    }
    value_label <- if (x$method == "power") {
      paste0(.icc_fmt(100 * x$value, 1), "%")
    } else {
      as.character(x$value)
    }
    value_caption <- switch(
      x$method,
      lower = "Required subjects for lower CI assurance",
      width = "Required subjects for CI half-width assurance",
      power = "Estimated assurance probability"
    )

    shiny::div(
      class = "metric-grid",
      .icc_metric_card("Result", value_label, value_caption, "accent"),
      .icc_metric_card("ICC model", x$mapping$icc_code, x$mapping$icc_full_name, "teal"),
      .icc_metric_card("Anticipated ICC", .icc_fmt(input$ss_rho, 2), paste0("k = ", input$ss_k), "green"),
      .icc_metric_card("Alpha", .icc_fmt(input$ss_alpha, 3), paste0("Assurance target = ", .icc_fmt(input$ss_assurance, 2)), "amber")
    )
  })

  output$power_plot <- shiny::renderPlot({
    x <- ss_result()
    shiny::req(isTRUE(x$ok))
    .icc_plot_power_curve(input, x)
  })

  output$ss_details <- shiny::renderTable({
    x <- ss_result()
    shiny::req(isTRUE(x$ok))
    .icc_ss_table(input, x)
  }, striped = TRUE, bordered = FALSE, spacing = "s")

  output$ss_export_preview <- shiny::renderTable({
    x <- ss_result()
    if (!isTRUE(x$ok)) {
      return(data.frame(Item = "Status", Value = x$error, check.names = FALSE))
    }
    .icc_ss_table(input, x)
  }, striped = TRUE, bordered = FALSE, spacing = "s")

  output$download_design <- shiny::downloadHandler(
    filename = function() paste0("icc-design-", Sys.Date(), ".csv"),
    content = function(file) {
      x <- ss_result()
      table <- if (isTRUE(x$ok)) {
        .icc_ss_table(input, x)
      } else {
        data.frame(Item = "Error", Value = x$error, check.names = FALSE)
      }
      utils::write.csv(table, file, row.names = FALSE)
    }
  )
}

.icc_sep_choices <- function() {
  c("Comma (,)" = ",", "Tab" = "\t", "Semicolon (;)" = ";", "Space" = " ")
}

.icc_read_uploaded <- function(file, header, sep, dec) {
  if (is.null(file) || is.null(file$datapath)) {
    return(NULL)
  }
  utils::read.table(
    file = file$datapath,
    header = isTRUE(header),
    sep = sep,
    dec = dec,
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
}

.icc_read_pasted <- function(text, header, sep, dec) {
  if (is.null(text) || !nzchar(trimws(text))) {
    return(NULL)
  }
  con <- textConnection(text)
  on.exit(close(con), add = TRUE)
  utils::read.table(
    file = con,
    header = isTRUE(header),
    sep = sep,
    dec = dec,
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
}

.icc_example_data <- function() {
  env <- new.env(parent = emptyenv())
  loaded <- tryCatch({
    utils::data("icc_data", package = "ICCDesign", envir = env)
    exists("icc_data", envir = env, inherits = FALSE)
  }, error = function(e) {
    FALSE
  })

  if (!loaded && file.exists("data/icc_data.rda")) {
    load("data/icc_data.rda", envir = env)
    loaded <- exists("icc_data", envir = env, inherits = FALSE)
  }
  if (!loaded) {
    stop("The built-in icc_data dataset could not be loaded.", call. = FALSE)
  }

  as.data.frame(env$icc_data, check.names = FALSE)
}

.icc_design_args <- function(input, prefix = "") {
  id <- function(x) if (nzchar(prefix)) paste0(prefix, "_", x) else x
  same <- identical(input[[id("same_raters")]], "TRUE")

  list(
    same_raters = same,
    rater_effect = if (same) input[[id("rater_effect")]] else NULL,
    rating_type = input[[id("rating_type")]],
    agreement_type = if (same) input[[id("agreement_type")]] else NULL
  )
}

.icc_alert <- function(title, message, type = c("success", "warning", "error")) {
  type <- match.arg(type)
  shiny::div(
    class = paste("notice", paste0("notice-", type)),
    shiny::strong(title),
    shiny::span(message)
  )
}

.icc_empty_state <- function(message) {
  shiny::div(class = "empty-state", message)
}

.icc_metric_card <- function(label, value, caption = NULL, tone = "accent") {
  shiny::div(
    class = paste("metric-card", paste0("metric-", tone)),
    shiny::span(class = "metric-label", label),
    shiny::strong(class = "metric-value", value),
    shiny::span(class = "metric-caption", .icc_or(caption, ""))
  )
}

.icc_named_list_table <- function(x) {
  data.frame(
    Metric = names(x),
    Value = vapply(x, function(value) {
      if (is.null(value)) {
        "NA"
      } else {
        .icc_fmt(value, 4)
      }
    }, character(1)),
    check.names = FALSE
  )
}

.icc_f_test_table <- function(icc_result) {
  tests <- list("ICC = 0" = icc_result$F_test_null)
  if (!is.null(icc_result$F_test_rho0)) {
    tests[["ICC = rho0"]] <- icc_result$F_test_rho0
  }

  rows <- lapply(names(tests), function(name) {
    test <- tests[[name]]
    data.frame(
      Hypothesis = name,
      F_stat = .icc_fmt(test$F_stat, 4),
      df1 = .icc_fmt(test$df1, 0),
      df2 = .icc_fmt(test$df2, 2),
      p_value = .icc_p_value(test$p_value),
      check.names = FALSE
    )
  })
  do.call(rbind, rows)
}

.icc_ss_table <- function(input, x) {
  method_label <- switch(
    x$method,
    lower = "Lower CI assurance",
    width = "CI half-width assurance",
    power = "Power for fixed n"
  )
  result_value <- if (x$method == "power") {
    paste0(.icc_fmt(100 * x$value, 2), "%")
  } else {
    as.character(x$value)
  }

  data.frame(
    Item = c(
      "Method",
      "Result",
      "Mapped ICC type",
      "Anticipated ICC",
      "Raters per subject",
      "Alpha",
      "Assurance target"
    ),
    Value = c(
      method_label,
      result_value,
      x$mapping$icc_full_name,
      .icc_fmt(input$ss_rho, 3),
      input$ss_k,
      .icc_fmt(input$ss_alpha, 3),
      .icc_fmt(input$ss_assurance, 3)
    ),
    check.names = FALSE
  )
}

.icc_selection_guide <- function() {
  data.frame(
    Same_raters = c("No", "No", "Yes", "Yes", "Yes", "Yes"),
    Rater_effect = c("Not specified", "Not specified", "Random", "Random", "Fixed", "Fixed"),
    Agreement = c("Absolute", "Absolute", "Absolute", "Absolute", "Consistency", "Consistency"),
    Rating = c("Single", "Average", "Single", "Average", "Single", "Average"),
    ICC_type = c("ICC(1,1)", "ICC(1,k)", "ICC(2,1)", "ICC(2,k)", "ICC(3,1)", "ICC(3,k)"),
    check.names = FALSE
  )
}

.icc_plot_interval <- function(point, lower, upper) {
  x_min <- min(-0.2, point - 0.05, lower, na.rm = TRUE)
  x_max <- 1
  oldpar <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(oldpar), add = TRUE)
  graphics::par(mar = c(4, 2, 2, 1), bg = "white")
  graphics::plot(
    NA,
    xlim = c(x_min, x_max),
    ylim = c(0.7, 1.3),
    xlab = "ICC value",
    ylab = "",
    xaxt = "n",
    yaxt = "n",
    bty = "n",
    main = ""
  )
  graphics::rect(x_min, 0.7, 0.5, 1.3, col = "#f7d6d0", border = NA)
  graphics::rect(0.5, 0.7, 0.75, 1.3, col = "#f4e7bd", border = NA)
  graphics::rect(0.75, 0.7, 0.9, 1.3, col = "#d7eadb", border = NA)
  graphics::rect(0.9, 0.7, x_max, 1.3, col = "#cfe8e5", border = NA)
  axis_at <- c(x_min, 0, 0.5, 0.75, 0.9, 1)
  axis_at <- axis_at[axis_at >= x_min & axis_at <= x_max]
  graphics::axis(
    1,
    at = axis_at,
    labels = vapply(axis_at, .icc_fmt, character(1), digits = 2),
    cex.axis = 0.82
  )
  graphics::abline(v = c(0.5, 0.75, 0.9), col = "#6b7280", lty = 3)
  graphics::segments(lower, 1, upper, 1, lwd = 5, col = "#256d75")
  graphics::points(point, 1, pch = 19, cex = 1.8, col = "#d85c48")
  graphics::text(
    x = c(0.25, 0.625, 0.825, 0.95),
    y = 1.22,
    labels = c("Poor", "Moderate", "Good", "Excellent"),
    cex = 0.82,
    col = "#374151"
  )
}

.icc_plot_power_curve <- function(input, x) {
  method <- if (x$method == "power") input$ss_power_method else x$method
  n_current <- if (x$method == "power") input$ss_n else x$value
  n_max <- max(12, ceiling(n_current * 1.8), n_current + 10)
  n_seq <- seq(2, n_max)

  powers <- vapply(n_seq, function(n) {
    tryCatch(
      do.call(
        icc_power,
        c(
          list(
            n = n,
            rho = input$ss_rho,
            rho0 = if (method == "lower") {
              if (x$method == "power") input$ss_power_rho0 else input$ss_lower_rho0
            } else {
              NULL
            },
            omega = if (method == "width") {
              if (x$method == "power") input$ss_power_omega else input$ss_width_omega
            } else {
              NULL
            },
            k = input$ss_k,
            alpha = input$ss_alpha,
            method = method,
            verbose = FALSE
          ),
          x$design
        )
      ),
      error = function(e) NA_real_
    )
  }, numeric(1))

  oldpar <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(oldpar), add = TRUE)
  graphics::par(mar = c(4, 4, 2, 1), bg = "white")
  graphics::plot(
    n_seq,
    powers,
    type = "l",
    lwd = 3,
    col = "#256d75",
    ylim = c(0, 1),
    xlab = "Subjects",
    ylab = "Assurance probability",
    bty = "n"
  )
  graphics::grid(col = "#e5e7eb")
  graphics::abline(h = input$ss_assurance, col = "#d85c48", lty = 2, lwd = 2)
  graphics::abline(v = n_current, col = "#374151", lty = 3, lwd = 2)
  graphics::points(n_current, powers[which.min(abs(n_seq - n_current))], pch = 19, col = "#d85c48", cex = 1.4)
}

.icc_fmt <- function(x, digits = 3) {
  if (is.null(x) || length(x) == 0 || is.na(x)) {
    return("NA")
  }
  formatC(as.numeric(x), format = "f", digits = digits)
}

.icc_p_value <- function(x) {
  if (is.null(x) || length(x) == 0 || is.na(x)) {
    return("NA")
  }
  format.pval(x, digits = 4, eps = 0.0001)
}

.icc_or <- function(x, fallback) {
  if (is.null(x) || length(x) == 0 || !nzchar(x)) {
    fallback
  } else {
    x
  }
}

.icc_app_css <- function() {
  "
  :root {
    --ink: #1f2937;
    --muted: #6b7280;
    --line: #dbe3ea;
    --surface: #ffffff;
    --surface-soft: #f6f8fb;
    --teal: #256d75;
    --green: #3f7d58;
    --amber: #a66b12;
    --coral: #d85c48;
    --danger: #b42318;
  }

  body {
    background: var(--surface-soft);
    color: var(--ink);
    font-family: Inter, -apple-system, BlinkMacSystemFont, 'Segoe UI', sans-serif;
    letter-spacing: 0;
  }

  .container-fluid {
    padding: 0;
  }

  .app-shell {
    max-width: 1440px;
    margin: 0 auto;
    padding: 24px;
  }

  .topbar {
    display: flex;
    align-items: center;
    justify-content: space-between;
    gap: 18px;
    padding: 18px 20px;
    background: var(--surface);
    border: 1px solid var(--line);
    border-radius: 8px;
    box-shadow: 0 14px 34px rgba(31, 41, 55, 0.06);
    margin-bottom: 16px;
  }

  .brand-block {
    display: flex;
    align-items: center;
    gap: 14px;
  }

  .brand-mark {
    width: 48px;
    height: 48px;
    border-radius: 8px;
    display: grid;
    place-items: center;
    background: var(--teal);
    color: #ffffff;
    font-weight: 800;
  }

  h1, h2, h3 {
    margin-top: 0;
    letter-spacing: 0;
  }

  h1 {
    font-size: 28px;
    margin-bottom: 3px;
    font-weight: 750;
  }

  h2 {
    font-size: 18px;
    margin-bottom: 8px;
    font-weight: 720;
  }

  h3 {
    font-size: 16px;
    margin-bottom: 12px;
    font-weight: 700;
  }

  .brand-block p,
  .muted {
    margin: 0;
    color: var(--muted);
  }

  .topbar-actions {
    display: flex;
    flex-wrap: wrap;
    gap: 8px;
    justify-content: flex-end;
  }

  .status-pill {
    border: 1px solid var(--line);
    color: var(--teal);
    background: #f0f7f6;
    padding: 7px 10px;
    border-radius: 8px;
    font-size: 12px;
    font-weight: 700;
  }

  .nav-tabs {
    border-bottom: 1px solid var(--line);
    margin-bottom: 16px;
  }

  .nav-tabs > li > a {
    color: var(--muted);
    border-radius: 8px 8px 0 0;
    font-weight: 700;
  }

  .nav-tabs > li.active > a,
  .nav-tabs > li.active > a:focus,
  .nav-tabs > li.active > a:hover {
    color: var(--teal);
    border-color: var(--line) var(--line) var(--surface);
  }

  .workspace-grid {
    display: grid;
    grid-template-columns: minmax(300px, 380px) minmax(0, 1fr);
    gap: 16px;
    align-items: start;
  }

  .guide-grid {
    display: grid;
    grid-template-columns: repeat(2, minmax(0, 1fr));
    gap: 16px;
  }

  .control-panel,
  .panel,
  .metric-card {
    background: var(--surface);
    border: 1px solid var(--line);
    border-radius: 8px;
    box-shadow: 0 12px 28px rgba(31, 41, 55, 0.05);
  }

  .control-panel {
    padding: 18px;
    position: sticky;
    top: 12px;
  }

  .panel {
    padding: 16px;
    overflow: hidden;
  }

  .panel-heading {
    display: flex;
    align-items: center;
    justify-content: space-between;
    gap: 12px;
    margin-bottom: 8px;
  }

  .result-stack {
    display: grid;
    gap: 16px;
  }

  .two-col,
  .metric-grid {
    display: grid;
    grid-template-columns: repeat(2, minmax(0, 1fr));
    gap: 16px;
  }

  .metric-grid {
    grid-template-columns: repeat(4, minmax(0, 1fr));
  }

  .metric-card {
    padding: 14px;
    min-height: 112px;
    border-top: 4px solid var(--teal);
  }

  .metric-accent {
    border-top-color: var(--coral);
  }

  .metric-teal {
    border-top-color: var(--teal);
  }

  .metric-green {
    border-top-color: var(--green);
  }

  .metric-amber {
    border-top-color: var(--amber);
  }

  .metric-label,
  .metric-caption {
    display: block;
    color: var(--muted);
    font-size: 12px;
    font-weight: 700;
  }

  .metric-value {
    display: block;
    font-size: 28px;
    line-height: 1.1;
    margin: 8px 0;
    overflow-wrap: anywhere;
  }

  .metric-caption {
    font-weight: 600;
    line-height: 1.35;
  }

  .input-grid {
    display: grid;
    grid-template-columns: repeat(2, minmax(0, 1fr));
    gap: 10px 12px;
  }

  .form-group {
    margin-bottom: 12px;
  }

  label {
    font-size: 12px;
    color: var(--muted);
    font-weight: 700;
  }

  .form-control,
  .selectize-input,
  textarea {
    border-color: var(--line);
    border-radius: 8px;
    box-shadow: none;
  }

  .form-control:focus,
  .selectize-input.focus {
    border-color: var(--teal);
    box-shadow: 0 0 0 3px rgba(37, 109, 117, 0.12);
  }

  .primary-action,
  .btn-default {
    border-radius: 8px;
    font-weight: 700;
  }

  .primary-action {
    width: 100%;
    margin-top: 8px;
    background: var(--teal);
    color: #ffffff;
    border: 1px solid var(--teal);
  }

  .primary-action:hover,
  .primary-action:focus {
    background: #1f5e65;
    color: #ffffff;
    border-color: #1f5e65;
  }

  .notice {
    display: flex;
    gap: 10px;
    align-items: flex-start;
    padding: 12px 14px;
    border-radius: 8px;
    border: 1px solid var(--line);
    background: var(--surface);
  }

  .notice strong {
    white-space: nowrap;
  }

  .notice-success {
    border-color: #b9ddc5;
    background: #f1f8f3;
  }

  .notice-warning {
    border-color: #f1d184;
    background: #fff8e8;
  }

  .notice-error {
    border-color: #f0b8b1;
    background: #fff1ef;
    color: var(--danger);
  }

  .empty-state {
    padding: 22px;
    text-align: center;
    color: var(--muted);
    border: 1px dashed var(--line);
    border-radius: 8px;
    background: var(--surface);
  }

  table {
    width: 100%;
    font-size: 13px;
  }

  pre {
    border: 1px solid var(--line);
    background: #fbfcfe;
    border-radius: 8px;
    color: var(--ink);
    white-space: pre-wrap;
  }

  @media (max-width: 1100px) {
    .workspace-grid,
    .guide-grid,
    .two-col,
    .metric-grid {
      grid-template-columns: 1fr;
    }

    .control-panel {
      position: static;
    }

    .topbar {
      align-items: flex-start;
      flex-direction: column;
    }
  }

  @media (max-width: 640px) {
    .app-shell {
      padding: 12px;
    }

    .input-grid {
      grid-template-columns: 1fr;
    }

    .metric-value {
      font-size: 23px;
    }
  }
  "
}

# File: main-interface.R
# Description: Top-level user interface functions for ICC analysis
# Author: [Ziyu Liu]
# Date: [2026-04-27]
# Dependencies: icc_preprocess_data, icc_check_design, and core ICC functions

#==============================================================================#
# Function 1: icc_map_design_to_icc
# Core decision mapping: 4 design questions -> ICC function
#==============================================================================#
#' Map Design Parameters to ICC Type
#'
#' @description
#' Maps user's 4 design questions to the corresponding ICC core calculation
#' function, full name, and any necessary warnings or tips.
#'
#' @param same_raters Logical. Are all subjects measured by the same group of raters?
#' @param rater_effect Character. "random" or "fixed". Ignored if \code{same_raters = FALSE}.
#' @param rating_type Character. "single" (single rating) or "average" (average of k ratings).
#' @param agreement_type Character. "absolute" (absolute agreement) or "consistency" (consistency).
#'   Ignored if \code{same_raters = FALSE}.
#'
#' @return A named list containing:
#' \describe{
#'   \item{icc_func_name}{Character. Name of the core ICC calculation function.}
#'   \item{icc_full_name}{Character. Full name of the ICC type.}
#'   \item{icc_code}{Character. ICC code (e.g., "1,1").}
#'   \item{warning_msg}{Character or NULL. Warning message for not recommended combinations.}
#'   \item{tip_msg}{Character or NULL. Tip message for automatically mapped combinations.}
#' }
#'
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
#' intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
icc_map_design_to_icc <- function(same_raters, rater_effect, rating_type, agreement_type) {
  # Initialize output
  result <- list(
    icc_func_name = NULL,
    icc_full_name = NULL,
    icc_code = NULL,
    warning_msg = NULL,
    tip_msg = NULL
  )

  #----------------------------------------------------------------------------#
  # Branch 1: One-way random effects (different raters per subject)
  #----------------------------------------------------------------------------#
  if (!same_raters) {
    if (rating_type == "single") {
      result$icc_func_name <- "icc_calc_1_1"
      result$icc_full_name <- "ICC(1,1) One-way random, single rating, absolute agreement"
      result$icc_code <- "1,1"
    } else if (rating_type == "average") {
      result$icc_func_name <- "icc_calc_1_k"
      result$icc_full_name <- "ICC(1,k) One-way random, average of k ratings, absolute agreement"
      result$icc_code <- "1,k"
    }
    return(result)
  }

  #----------------------------------------------------------------------------#
  # Branch 2: Two-way models (same raters for all subjects)
  #----------------------------------------------------------------------------#
  if (same_raters) {
    # Case A: Random effects + Absolute agreement
    if (rater_effect == "random" && agreement_type == "absolute") {
      if (rating_type == "single") {
        result$icc_func_name <- "icc_calc_2_1"
        result$icc_full_name <- "ICC(2,1) Two-way random, single rating, absolute agreement"
        result$icc_code <- "2,1"
      } else if (rating_type == "average") {
        result$icc_func_name <- "icc_calc_2_k"
        result$icc_full_name <- "ICC(2,k) Two-way random, average of k ratings, absolute agreement"
        result$icc_code <- "2,k"
      }
    }

    # Case B: Fixed effects + Consistency
    else if (rater_effect == "fixed" && agreement_type == "consistency") {
      if (rating_type == "single") {
        result$icc_func_name <- "icc_calc_3_1"
        result$icc_full_name <- "ICC(3,1) Two-way mixed, single rating, consistency"
        result$icc_code <- "3,1"
      } else if (rating_type == "average") {
        result$icc_func_name <- "icc_calc_3_k"
        result$icc_full_name <- "ICC(3,k) Two-way mixed, average of k ratings, consistency"
        result$icc_code <- "3,k"
      }
    }

    # Case C: Random effects + Consistency (auto-map to ICC(3,1)/ICC(3,k))
    else if (rater_effect == "random" && agreement_type == "consistency") {
      if (rating_type == "single") {
        result$icc_func_name <- "icc_calc_2_1_consistency"
        result$icc_full_name <- "ICC(2,1,consistency) Two-way random, single rating, consistency (mapped to ICC(3,1))"
        result$icc_code <- "2,1,consistency"
      } else if (rating_type == "average") {
        result$icc_func_name <- "icc_calc_2_k_consistency"
        result$icc_full_name <- "ICC(2,k,consistency) Two-way random, average of k ratings, consistency (mapped to ICC(3,k))"
        result$icc_code <- "2,k,consistency"
      }
      result$tip_msg <- paste(
        "Random effects + consistency combination specified.",
        "In practice, this is automatically mapped to the mixed effects model (ICC(3,x)).",
        "See framework documentation for details."
      )
    }

    # Case D: Fixed effects + Absolute agreement (NOT RECOMMENDED)
    else if (rater_effect == "fixed" && agreement_type == "absolute") {
      if (rating_type == "single") {
        result$icc_func_name <- "icc_calc_3_1_absolute"
        result$icc_full_name <- "ICC(3,1,absolute) Two-way mixed, single rating, absolute agreement (NOT RECOMMENDED)"
        result$icc_code <- "3,1,absolute"
      } else if (rating_type == "average") {
        result$icc_func_name <- "icc_calc_3_k_absolute"
        result$icc_full_name <- "ICC(3,k,absolute) Two-way mixed, average of k ratings, absolute agreement (NOT RECOMMENDED)"
        result$icc_code <- "3,k,absolute"
      }
      result$warning_msg <- paste(
        "WARNING: Fixed effects + absolute agreement combination is rarely used",
        "and difficult to interpret. It is strongly recommended to use",
        "the random effects + absolute agreement model (ICC(2,x)) instead.",
        "See framework documentation for details."
      )
    }
  }

  return(result)
}

#==============================================================================#
# Function 2: icc_evaluate
# Evaluate ICC reliability based on Koo & Li (2016) criteria
#==============================================================================#
#' Evaluate ICC Reliability
#'
#' @description
#' Evaluates the reliability of an ICC result based on the 95% confidence
#' interval lower bound, following the criteria of Koo & Li (2016).
#'
#' @param icc_result List. Output from a core ICC calculation function.
#'
#' @return A named list with elements:
#' \itemize{
#'   \item icc_code: Character. ICC code.
#'   \item point_est: Numeric. ICC point estimate.
#'   \item ci_lower: Numeric. 95% CI lower bound.
#'   \item rating_en: Character. English reliability rating.
#'   \item explanation: Character. Interpretation text.
#' }
#'
#' @keywords internal
#' @references
#' Koo, T. K., & Li, M. Y. (2016). A guideline of selecting and reporting
#' intraclass correlation coefficients for reliability research. Journal of
#' Chiropractic Medicine, 15(2), 155-163.
icc_evaluate <- function(icc_result) {
  # Extract values
  icc_code <- icc_result$icc_code
  point_est <- icc_result$point_est
  ci_lower <- icc_result$ci_lower
  ci_upper <- icc_result$ci_upper

  # Determine rating based on CI lower bound
  if (ci_lower < 0.5) {
    rating_en <- "Poor"
  } else if (ci_lower >= 0.5 && ci_lower < 0.75) {
    rating_en <- "Moderate"
  } else if (ci_lower >= 0.75 && ci_lower < 0.9) {
    rating_en <- "Good"
  } else {
    rating_en <- "Excellent"
  }

  # Build explanation
  explanation <- paste0(
    "Based on the 95% CI lower bound (", sprintf("%.3f", ci_lower), "), ",
    "the reliability is rated as '", rating_en, "'. "
  )

  # Add cross-interval warning if needed
  if (ci_lower < 0.75 && ci_upper >= 0.75) {
    explanation <- paste0(
      explanation,
      "Note: The confidence interval spans multiple rating categories, ",
      "indicating uncertainty in the reliability estimate. ",
      "Consider increasing the sample size for a more precise estimate."
    )
  }

  # Return result (removed rating_cn for CRAN compliance)
  list(
    icc_code = icc_code,
    point_est = point_est,
    ci_lower = ci_lower,
    rating_en = rating_en,
    explanation = explanation
  )
}

#==============================================================================#
# Function 3: icc_generate_report
# Generate standardized publication-ready report
#==============================================================================#
#' Generate ICC Report
#'
#' @description
#' Generates a standardized, publication-ready report of ICC results,
#' supporting text, Markdown, and HTML formats.
#'
#' @param icc_result List. Output from a core ICC calculation function.
#' @param evaluation List. Output from \code{icc_evaluate}.
#' @param format Character. Output format: "text" (default), "markdown", or "html".
#'
#' @return Character. Formatted report text.
#'
#' @keywords internal
icc_generate_report <- function(icc_result, evaluation, format = "text") {
  # Extract all necessary information (removed rating_cn)
  icc_full_name <- icc_result$icc_type
  icc_code <- icc_result$icc_code
  point_est <- icc_result$point_est
  ci_level <- icc_result$ci_level
  ci_lower <- icc_result$ci_lower
  ci_upper <- icc_result$ci_upper
  f_test_null <- icc_result$F_test_null
  rating_en <- evaluation$rating_en
  explanation <- evaluation$explanation

  # Build report components
  header <- "=== ICC Analysis Report ==="
  model_section <- paste0("Model Type: ", icc_full_name)
  result_section <- paste0(
    "Results:\n",
    "  ICC Point Estimate: ", sprintf("%.4f", point_est), "\n",
    "  ", sprintf("%.0f%%", ci_level * 100), " Confidence Interval: [",
    sprintf("%.4f", ci_lower), ", ", sprintf("%.4f", ci_upper), "]"
  )

  test_section <- if (!is.null(f_test_null)) {

    # Format p-value: if <0.0001, show "<0.0001", else 4 decimal places
    p_val_formatted <- if (!is.null(f_test_null$p_value)) {
      if (f_test_null$p_value < 0.0001) {
        "<0.0001"
      } else {
        sprintf("%.4f", f_test_null$p_value)
      }
    } else {
      "N/A"
    }

    paste0(
      "Hypothesis Test (H0: ICC = 0):\n",
      "  F-statistic: ", sprintf("%.4f", f_test_null$F_stat), "\n",
      "  DF1: ", f_test_null$df1, ", DF2: ", f_test_null$df2, "\n",
      "  p-value: ", p_val_formatted
    )
  } else {
    "Hypothesis Test: Not available"
  }

  rating_section <- paste0(
    "Reliability Rating:\n",
    "  Category: ", rating_en, "\n",
    "  Interpretation: ", explanation
  )

  # Combine based on format (removed all rating_cn references)
  if (format == "text") {
    report <- paste0(
      header, "\n\n",
      model_section, "\n\n",
      result_section, "\n\n",
      test_section, "\n\n",
      rating_section
    )
  } else if (format == "markdown") {
    report <- paste0(
      "# ICC Analysis Report\n\n",
      "## Model Type\n", icc_full_name, "\n\n",
      "## Results\n",
      "- ICC Point Estimate: ", sprintf("%.4f", point_est), "\n",
      "- ", sprintf("%.0f%%", ci_level * 100), " CI: [",
      sprintf("%.4f", ci_lower), ", ", sprintf("%.4f", ci_upper), "]\n\n",
      "## Hypothesis Test\n",
      "- F-statistic: ", sprintf("%.4f", f_test_null$F_stat), "\n",
      "- DF: (", f_test_null$df1, ", ", f_test_null$df2, ")\n",
      "- p-value: ", if (!is.null(f_test_null$p_value)) sprintf("%.4f", f_test_null$p_value) else "N/A", "\n\n",
      "## Reliability Rating\n",
      "- **", rating_en, "**\n",
      "- ", explanation
    )
  } else if (format == "html") {
    report <- paste0(
      "<h1>ICC Analysis Report</h1>",
      "<h2>Model Type</h2><p>", icc_full_name, "</p>",
      "<h2>Results</h2>",
      "<ul>",
      "<li>ICC Point Estimate: <strong>", sprintf("%.4f", point_est), "</strong></li>",
      "<li>", sprintf("%.0f%%", ci_level * 100), " CI: [",
      sprintf("%.4f", ci_lower), ", ", sprintf("%.4f", ci_upper), "]</li>",
      "</ul>",
      "<h2>Hypothesis Test</h2>",
      "<ul>",
      "<li>F-statistic: ", sprintf("%.4f", f_test_null$F_stat), "</li>",
      "<li>DF: (", f_test_null$df1, ", ", f_test_null$df2, ")</li>",
      "<li>p-value: ", if (!is.null(f_test_null$p_value)) sprintf("%.4f", f_test_null$p_value) else "N/A", "</li>",
      "</ul>",
      "<h2>Reliability Rating</h2>",
      "<p><strong>", rating_en, "</strong></p>",
      "<p>", explanation, "</p>"
    )
  }

  return(report)
}

#==============================================================================#
# Function 4: icc_calc (TOP-LEVEL MAIN FUNCTION)
# Complete end-to-end ICC analysis workflow
#==============================================================================#
#' Calculate Intraclass Correlation Coefficient (ICC)
#'
#' @description
#' Top-level main function for complete ICC analysis. Users only need to
#' provide raw data and answer 4 design questions. The function automatically
#' handles data preprocessing, parameter validation, ICC type mapping,
#' core calculation, reliability evaluation, and report generation.
#'
#' @param data Data frame or matrix. Raw data where rows = subjects,
#'   columns = raters/measurements.
#' @param same_raters Logical. Are all subjects measured by the same group
#'   of raters?
#' @param rater_effect Character. "random" or "fixed". Ignored if
#'   \code{same_raters = FALSE}.
#' @param rating_type Character. "single" (single rating) or "average"
#'   (average of k ratings).
#' @param agreement_type Character. "absolute" (absolute agreement) or
#'   "consistency" (consistency). Ignored if \code{same_raters = FALSE}.
#' @param alpha Numeric. Significance level for confidence interval,
#'   default 0.05.
#' @param rho0 Numeric. Optional null hypothesis value for non-zero test,
#'   default NULL.
#' @param interaction Logical. Whether to include interaction term in
#'   two-way models, default TRUE.
#' @param na.rm Logical. Whether to automatically remove rows with missing
#'   values, default TRUE.
#' @param verbose Logical. Whether to emit warnings and tips, default TRUE.
#'
#' @return A named list containing:
#' \describe{
#'   \item{data_summary}{List. Data preprocessing summary.}
#'   \item{icc_result}{List. Full ICC calculation results.}
#'   \item{evaluation}{List. Reliability evaluation results.}
#'   \item{report}{Character. Standardized report text.}
#'   \item{warning_msg}{Character or NULL. Warning message.}
#'   \item{tip_msg}{Character or NULL. Tip message.}
#' }
#'
#' @export
#' @references
#' McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
#' intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
#'
#' Koo, T. K., & Li, M. Y. (2016). A guideline of selecting and reporting
#' intraclass correlation coefficients for reliability research. Journal of
#' Chiropractic Medicine, 15(2), 155-163.
#'
#' @examples
#' # Example 0: Use built-in example dataset
#' data(icc_data)
#' result <- icc_calc(icc_data, same_raters = TRUE, rater_effect = "random",
#'                    rating_type = "single", agreement_type = "absolute")
#' # Example 1: One-way random effects, single rating (ICC(1,1))
#' data <- matrix(rnorm(100), nrow = 20, ncol = 5)
#' result <- icc_calc(data, same_raters = FALSE, rating_type = "single")
#'
#' # Example 2: Two-way random effects, average rating, absolute agreement (ICC(2,k))
#' result <- icc_calc(data, same_raters = TRUE, rater_effect = "random",
#'                    rating_type = "average", agreement_type = "absolute")
#'
#' # Example 3: Two-way mixed effects, single rating, consistency (ICC(3,1))
#' result <- icc_calc(data, same_raters = TRUE, rater_effect = "fixed",
#'                    rating_type = "single", agreement_type = "consistency")
#'
#' # Example 4: Special scenario - automatic mapping with tip
#' # Random effects + consistency (automatically mapped to ICC(3,1))
#' result <- icc_calc(data, same_raters = TRUE, rater_effect = "random",
#'                    rating_type = "single", agreement_type = "consistency")
#'
#' # Example 5: Special scenario - not recommended combination with warning
#' # Fixed effects + absolute agreement (NOT RECOMMENDED)
#' result <- icc_calc(data, same_raters = TRUE, rater_effect = "fixed",
#'                    rating_type = "average", agreement_type = "absolute")
#'
#' # Example 6: Advanced parameters - custom alpha and non-zero test
#' result <- icc_calc(data, same_raters = TRUE, rater_effect = "random",
#'                    rating_type = "single", agreement_type = "absolute",
#'                    alpha = 0.1, rho0 = 0.6, verbose = FALSE)
#'
#' # Example 7: Extract specific results
#' result$icc_result$point_est  # ICC point estimate
#' result$evaluation$rating_en  # Reliability rating
#' result$report               # Full text report
icc_calc <- function(data, same_raters, rater_effect = NULL,
                     rating_type, agreement_type = NULL,
                     alpha = 0.05, rho0 = NULL, interaction = TRUE,
                     na.rm = TRUE, verbose = TRUE) {
  #----------------------------------------------------------------------------#
  # Step 1: Data Preprocessing
  #----------------------------------------------------------------------------#
  data_summary <- icc_preprocess_data(data, na.rm = na.rm)
  if (!is.null(data_summary$error_msg)) {
    stop(data_summary$error_msg, call. = FALSE)
  }
  data_matrix <- data_summary$data_matrix

  #----------------------------------------------------------------------------#
  # Step 2: Design Parameter Validation
  #----------------------------------------------------------------------------#
  design_check <- icc_check_design(
    same_raters = same_raters,
    rater_effect = rater_effect,
    rating_type = rating_type,
    agreement_type = agreement_type,
    k = data_summary$k
  )
  if (!design_check$is_valid) {
    stop(design_check$error_msg, call. = FALSE)
  }

  #----------------------------------------------------------------------------#
  # Step 3: Map Design to ICC Function
  #----------------------------------------------------------------------------#
  mapping <- icc_map_design_to_icc(
    same_raters = same_raters,
    rater_effect = rater_effect,
    rating_type = rating_type,
    agreement_type = agreement_type
  )

  warning_msg <- mapping$warning_msg
  tip_msg <- mapping$tip_msg

  #----------------------------------------------------------------------------#
  # Step 4: Core ICC Calculation
  #----------------------------------------------------------------------------#
  # Dynamically get the function
  icc_func <- get(
    mapping$icc_func_name,
    envir = environment(icc_calc),
    mode = "function",
    inherits = TRUE
  )

  # Call the function
  icc_result <- icc_func(
    data_matrix = data_matrix,
    alpha = alpha,
    rho0 = rho0,
    interaction = interaction
  )

  #----------------------------------------------------------------------------#
  # Step 5: Reliability Evaluation
  #----------------------------------------------------------------------------#
  evaluation <- icc_evaluate(icc_result)

  #----------------------------------------------------------------------------#
  # Step 6: Generate Report
  #----------------------------------------------------------------------------#
  report <- icc_generate_report(icc_result, evaluation, format = "text")

  #----------------------------------------------------------------------------#
  # Step 7: Messages (if verbose = TRUE)
  #----------------------------------------------------------------------------#
  if (verbose) {
    # Emit warning if exists
    if (!is.null(warning_msg)) {
      warning(warning_msg, call. = FALSE)
    }

    # Emit tip if exists
    if (!is.null(tip_msg)) {
      message("NOTE: ", tip_msg)
    }
  }

  #----------------------------------------------------------------------------#
  # Step 8: Return Complete Result
  #----------------------------------------------------------------------------#
  result <- list(
    data_summary = data_summary,
    icc_result = icc_result,
    evaluation = evaluation,
    report = report,
    warning_msg = warning_msg,
    tip_msg = tip_msg
  )

  # Set class for pretty printing
  class(result) <- "icc_result"

  return(result)
}

#' ICCDesign: Intraclass Correlation Coefficient Analysis and Study Planning
#'
#' @description
#' A comprehensive, user-friendly R package for intraclass correlation coefficient (ICC)
#' analysis and sample size planning. Implements the full McGraw & Wong (1996) framework
#' supporting all 10 ICC types, with an intuitive 4-question decision system that
#' eliminates the need for users to memorize ICC type codes.
#'
#' The package provides both a command-line interface for advanced users and a
#' fully interactive Shiny web application for point-and-click analysis. It also
#' includes automated reliability evaluation based on Koo & Li (2016) criteria,
#' publication-ready report generation, and rigorous sample size and power
#' calculation functions.
#'
#' @section Key Features:
#' \itemize{
#'   \item Full support for all 10 ICC types from McGraw & Wong (1996)
#'   \item Intuitive 4-question decision framework for ICC type selection
#'   \item Automated reliability rating and interpretation
#'   \item Publication-ready text, Markdown, and HTML reports
#'   \item Sample size calculation based on confidence interval lower bound or width
#'   \item Power analysis for existing study designs
#'   \item Interactive Shiny web application with data upload and visualization
#'   \item Comprehensive data validation and error handling
#' }
#'
#' @section Main Functions:
#' \describe{
#'   \item{\code{\link{icc_calc}}}{Top-level function for complete ICC analysis}
#'   \item{\code{\link{icc_sample_size}}}{Unified interface for sample size and power calculation}
#'   \item{\code{\link{run_icc_app}}}{Launch the interactive Shiny application}
#'   \item{\code{\link{icc_preprocess_data}}}{Data preprocessing and validation utility}
#' }
#'
#' @references
#' McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
#' intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
#'
#' Koo, T. K., & Li, M. Y. (2016). A guideline of selecting and reporting
#' intraclass correlation coefficients for reliability research. Journal of
#' Chiropractic Medicine, 15(2), 155-163.
#'
#' @name ICCDesign-package
#' @aliases ICCDesign
"_PACKAGE"

#' Example ICC Dataset
#'
#' A small, carefully constructed example dataset containing ratings from
#' 4 raters on 5 subjects. Designed to demonstrate all package functionality
#' with fast execution and predictable results.
#'
#' This dataset can be used to calculate all 10 ICC types supported by the
#' package, simply by changing the design parameters in \code{\link{icc_calc}}.
#'
#' @name icc_data
#' @format A numeric matrix with 5 rows (subjects) and 4 columns (raters).
#'   Row names are "Subject1" to "Subject5", column names are "Rater1" to "Rater4".
#'
#' @details
#' The dataset was simulated to have an approximate ICC of 0.8, which falls
#' into the "Good" reliability category according to Koo & Li (2016).
#'
#' @source Simulated data for demonstration purposes.
#'
#' @examples
#' data(icc_data)
#' head(icc_data)
NULL

# File: core-icc.R
# Description: Core calculation functions for all 10 ICC types
#              (6 common + 4 supplementary)
# Author: [Ziyu Liu]
# Date: [20260421]
# Dependencies: All functions rely on internal utility functions from
#               utils-data.R and utils-calc.R

#==============================================================================#
# Helper: Internal function to assemble standardized output (DRY principle)
# Not exported, only used within this file
#==============================================================================#
.assemble_icc_output <- function(icc_type_full, icc_type_code,
                                 anova_res, point_est, ci_res,
                                 f_test_null, f_test_rho0 = NULL,
                                 warning_msg = NULL, tip_msg = NULL) {
  list(
    icc_type = icc_type_full,
    icc_code = icc_type_code,
    point_est = point_est,
    ci_level = ci_res$ci_level,
    ci_lower = ci_res$ci_lower,
    ci_upper = ci_res$ci_upper,
    F_test_null = f_test_null,
    F_test_rho0 = f_test_rho0,
    anova_summary = anova_res[c("MSR", "MSC", "MSE", "MSW", "df1", "df2", "df3", "n", "k")],
    warning_msg = warning_msg,
    tip_msg = tip_msg
  )
}

#==============================================================================#
# ICC(1,1): One-way random effects, single rating, absolute agreement
#==============================================================================#
#' Calculate ICC(1,1)
#'
#' @description
#' Calculates the Intraclass Correlation Coefficient (ICC) for a one-way
#' random effects model using a single rater/measurement, focusing on
#' absolute agreement.
#'
#' @param data_matrix A standardized numeric matrix from \code{icc_preprocess_data}.
#'   Rows = subjects, columns = raters/measurements.
#' @param alpha Significance level for confidence interval, default 0.05.
#' @param rho0 Optional null hypothesis value for non-zero test, default NULL.
#' @param interaction Logical, whether to include interaction term (ignored for one-way).
#'
#' @return A standardized list containing ICC results, see package documentation for details.
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
#' intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
icc_calc_1_1 <- function(data_matrix, alpha = 0.05, rho0 = NULL, interaction = TRUE) {
  # 1. Calculate one-way ANOVA
  anova_res <- icc_calc_anova(data_matrix, model_type = "oneway")

  # 2. Calculate point estimate
  point_est <- icc_tool_point(anova_res, icc_type = "1,1")

  # 3. Calculate confidence interval
  ci_res <- icc_tool_ci(anova_res, icc_type = "1,1", point_est = point_est, alpha = alpha)

  # 4. Calculate F-test for H0: ICC = 0
  f_test_null <- icc_calc_f_test(anova_res, icc_type = "1,1", rho0 = 0, alpha = alpha)

  # 5. Calculate F-test for H0: ICC = rho0 (if provided)
  f_test_rho0 <- if (!is.null(rho0)) {
    icc_calc_f_test(anova_res, icc_type = "1,1", rho0 = rho0, alpha = alpha)
  } else {
    NULL
  }

  # 6. Assemble and return standardized output
  .assemble_icc_output(
    icc_type_full = "ICC(1,1) One-way random, single rating, absolute agreement",
    icc_type_code = "1,1",
    anova_res = anova_res,
    point_est = point_est,
    ci_res = ci_res,
    f_test_null = f_test_null,
    f_test_rho0 = f_test_rho0,
    warning_msg = NULL,
    tip_msg = NULL
  )
}

#==============================================================================#
# ICC(1,k): One-way random effects, average of k ratings, absolute agreement
#==============================================================================#
#' Calculate ICC(1,k)
#'
#' @description
#' Calculates the Intraclass Correlation Coefficient (ICC) for a one-way
#' random effects model using the average of k raters/measurements, focusing
#' on absolute agreement.
#'
#' @inheritParams icc_calc_1_1
#' @return A standardized list containing ICC results.
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996).
icc_calc_1_k <- function(data_matrix, alpha = 0.05, rho0 = NULL, interaction = TRUE) {
  anova_res <- icc_calc_anova(data_matrix, model_type = "oneway")
  point_est <- icc_tool_point(anova_res, icc_type = "1,k")
  ci_res <- icc_tool_ci(anova_res, icc_type = "1,k", point_est = point_est, alpha = alpha)
  f_test_null <- icc_calc_f_test(anova_res, icc_type = "1,k", rho0 = 0, alpha = alpha)
  f_test_rho0 <- if (!is.null(rho0)) icc_calc_f_test(anova_res, icc_type = "1,k", rho0 = rho0, alpha = alpha) else NULL

  .assemble_icc_output(
    icc_type_full = "ICC(1,k) One-way random, average of k ratings, absolute agreement",
    icc_type_code = "1,k",
    anova_res = anova_res,
    point_est = point_est,
    ci_res = ci_res,
    f_test_null = f_test_null,
    f_test_rho0 = f_test_rho0,
    warning_msg = NULL,
    tip_msg = NULL
  )
}

#==============================================================================#
# ICC(2,1): Two-way random effects, single rating, absolute agreement
#==============================================================================#
#' Calculate ICC(2,1)
#'
#' @description
#' Calculates the Intraclass Correlation Coefficient (ICC) for a two-way
#' random effects model using a single rater, focusing on absolute agreement.
#'
#' @inheritParams icc_calc_1_1
#' @return A standardized list containing ICC results.
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996).
icc_calc_2_1 <- function(data_matrix, alpha = 0.05, rho0 = NULL, interaction = TRUE) {
  anova_res <- icc_calc_anova(data_matrix, model_type = "twoway", interaction = interaction)
  point_est <- icc_tool_point(anova_res, icc_type = "2,1")
  ci_res <- icc_tool_ci(anova_res, icc_type = "2,1", point_est = point_est, alpha = alpha)
  f_test_null <- icc_calc_f_test(anova_res, icc_type = "2,1", rho0 = 0, alpha = alpha)
  f_test_rho0 <- if (!is.null(rho0)) icc_calc_f_test(anova_res, icc_type = "2,1", rho0 = rho0, alpha = alpha) else NULL

  .assemble_icc_output(
    icc_type_full = "ICC(2,1) Two-way random, single rating, absolute agreement",
    icc_type_code = "2,1",
    anova_res = anova_res,
    point_est = point_est,
    ci_res = ci_res,
    f_test_null = f_test_null,
    f_test_rho0 = f_test_rho0,
    warning_msg = NULL,
    tip_msg = NULL
  )
}

#==============================================================================#
# ICC(2,k): Two-way random effects, average of k ratings, absolute agreement
#==============================================================================#
#' Calculate ICC(2,k)
#'
#' @description
#' Calculates the Intraclass Correlation Coefficient (ICC) for a two-way
#' random effects model using the average of k raters, focusing on absolute agreement.
#'
#' @inheritParams icc_calc_1_1
#' @return A standardized list containing ICC results.
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996).
icc_calc_2_k <- function(data_matrix, alpha = 0.05, rho0 = NULL, interaction = TRUE) {
  anova_res <- icc_calc_anova(data_matrix, model_type = "twoway", interaction = interaction)
  point_est <- icc_tool_point(anova_res, icc_type = "2,k")
  ci_res <- icc_tool_ci(anova_res, icc_type = "2,k", point_est = point_est, alpha = alpha)
  f_test_null <- icc_calc_f_test(anova_res, icc_type = "2,k", rho0 = 0, alpha = alpha)
  f_test_rho0 <- if (!is.null(rho0)) icc_calc_f_test(anova_res, icc_type = "2,k", rho0 = rho0, alpha = alpha) else NULL

  .assemble_icc_output(
    icc_type_full = "ICC(2,k) Two-way random, average of k ratings, absolute agreement",
    icc_type_code = "2,k",
    anova_res = anova_res,
    point_est = point_est,
    ci_res = ci_res,
    f_test_null = f_test_null,
    f_test_rho0 = f_test_rho0,
    warning_msg = NULL,
    tip_msg = NULL
  )
}

#==============================================================================#
# ICC(3,1): Two-way mixed effects, single rating, consistency
#==============================================================================#
#' Calculate ICC(3,1)
#'
#' @description
#' Calculates the Intraclass Correlation Coefficient (ICC) for a two-way
#' mixed effects model using a single rater, focusing on consistency.
#'
#' @inheritParams icc_calc_1_1
#' @return A standardized list containing ICC results.
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996).
icc_calc_3_1 <- function(data_matrix, alpha = 0.05, rho0 = NULL, interaction = TRUE) {
  anova_res <- icc_calc_anova(data_matrix, model_type = "twoway", interaction = interaction)
  point_est <- icc_tool_point(anova_res, icc_type = "3,1")
  ci_res <- icc_tool_ci(anova_res, icc_type = "3,1", point_est = point_est, alpha = alpha)
  f_test_null <- icc_calc_f_test(anova_res, icc_type = "3,1", rho0 = 0, alpha = alpha)
  f_test_rho0 <- if (!is.null(rho0)) icc_calc_f_test(anova_res, icc_type = "3,1", rho0 = rho0, alpha = alpha) else NULL

  .assemble_icc_output(
    icc_type_full = "ICC(3,1) Two-way mixed, single rating, consistency",
    icc_type_code = "3,1",
    anova_res = anova_res,
    point_est = point_est,
    ci_res = ci_res,
    f_test_null = f_test_null,
    f_test_rho0 = f_test_rho0,
    warning_msg = NULL,
    tip_msg = NULL
  )
}

#==============================================================================#
# ICC(3,k): Two-way mixed effects, average of k ratings, consistency
#==============================================================================#
#' Calculate ICC(3,k)
#'
#' @description
#' Calculates the Intraclass Correlation Coefficient (ICC) for a two-way
#' mixed effects model using the average of k raters, focusing on consistency.
#'
#' @inheritParams icc_calc_1_1
#' @return A standardized list containing ICC results.
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996).
icc_calc_3_k <- function(data_matrix, alpha = 0.05, rho0 = NULL, interaction = TRUE) {
  anova_res <- icc_calc_anova(data_matrix, model_type = "twoway", interaction = interaction)
  point_est <- icc_tool_point(anova_res, icc_type = "3,k")
  ci_res <- icc_tool_ci(anova_res, icc_type = "3,k", point_est = point_est, alpha = alpha)
  f_test_null <- icc_calc_f_test(anova_res, icc_type = "3,k", rho0 = 0, alpha = alpha)
  f_test_rho0 <- if (!is.null(rho0)) icc_calc_f_test(anova_res, icc_type = "3,k", rho0 = rho0, alpha = alpha) else NULL

  .assemble_icc_output(
    icc_type_full = "ICC(3,k) Two-way mixed, average of k ratings, consistency",
    icc_type_code = "3,k",
    anova_res = anova_res,
    point_est = point_est,
    ci_res = ci_res,
    f_test_null = f_test_null,
    f_test_rho0 = f_test_rho0,
    warning_msg = NULL,
    tip_msg = NULL
  )
}

#==============================================================================#
# Supplementary ICC: ICC(2,1,consistency)
# Maps to ICC(3,1) in practice with a tip message
#==============================================================================#
#' Calculate ICC(2,1,consistency) [Supplementary]
#'
#' @description
#' Calculates ICC for two-way random effects, single rating, consistency.
#' Note: In practice, this maps to ICC(3,1) (mixed effects model).
#'
#' @inheritParams icc_calc_1_1
#' @return A standardized list containing ICC results.
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996).
icc_calc_2_1_consistency <- function(data_matrix, alpha = 0.05, rho0 = NULL, interaction = TRUE) {
  anova_res <- icc_calc_anova(data_matrix, model_type = "twoway", interaction = interaction)
  point_est <- icc_tool_point(anova_res, icc_type = "3,1") # Use same formula as ICC(3,1)
  ci_res <- icc_tool_ci(anova_res, icc_type = "3,1", point_est = point_est, alpha = alpha)
  f_test_null <- icc_calc_f_test(anova_res, icc_type = "3,1", rho0 = 0, alpha = alpha)
  f_test_rho0 <- if (!is.null(rho0)) icc_calc_f_test(anova_res, icc_type = "3,1", rho0 = rho0, alpha = alpha) else NULL

  tip_msg <- paste(
    "Random effects + consistency combination specified.",
    "In practice, this is automatically mapped to ICC(3,1) (mixed effects model).",
    "See framework documentation for details."
  )

  .assemble_icc_output(
    icc_type_full = "ICC(2,1,consistency) Two-way random, single rating, consistency (mapped to ICC(3,1))",
    icc_type_code = "2,1,consistency",
    anova_res = anova_res,
    point_est = point_est,
    ci_res = ci_res,
    f_test_null = f_test_null,
    f_test_rho0 = f_test_rho0,
    warning_msg = NULL,
    tip_msg = tip_msg
  )
}

#==============================================================================#
# Supplementary ICC: ICC(2,k,consistency)
# Maps to ICC(3,k) in practice with a tip message
#==============================================================================#
#' Calculate ICC(2,k,consistency) [Supplementary]
#'
#' @description
#' Calculates ICC for two-way random effects, average of k ratings, consistency.
#' Note: In practice, this maps to ICC(3,k) (mixed effects model).
#'
#' @inheritParams icc_calc_1_1
#' @return A standardized list containing ICC results.
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996).
icc_calc_2_k_consistency <- function(data_matrix, alpha = 0.05, rho0 = NULL, interaction = TRUE) {
  anova_res <- icc_calc_anova(data_matrix, model_type = "twoway", interaction = interaction)
  point_est <- icc_tool_point(anova_res, icc_type = "3,k")
  ci_res <- icc_tool_ci(anova_res, icc_type = "3,k", point_est = point_est, alpha = alpha)
  f_test_null <- icc_calc_f_test(anova_res, icc_type = "3,k", rho0 = 0, alpha = alpha)
  f_test_rho0 <- if (!is.null(rho0)) icc_calc_f_test(anova_res, icc_type = "3,k", rho0 = rho0, alpha = alpha) else NULL

  tip_msg <- paste(
    "Random effects + consistency combination specified.",
    "In practice, this is automatically mapped to ICC(3,k) (mixed effects model).",
    "See framework documentation for details."
  )

  .assemble_icc_output(
    icc_type_full = "ICC(2,k,consistency) Two-way random, average of k ratings, consistency (mapped to ICC(3,k))",
    icc_type_code = "2,k,consistency",
    anova_res = anova_res,
    point_est = point_est,
    ci_res = ci_res,
    f_test_null = f_test_null,
    f_test_rho0 = f_test_rho0,
    warning_msg = NULL,
    tip_msg = tip_msg
  )
}

#==============================================================================#
# Supplementary ICC: ICC(3,1,absolute)
# Not recommended, provides a warning message
#==============================================================================#
#' Calculate ICC(3,1,absolute) [Supplementary]
#'
#' @description
#' Calculates ICC for two-way mixed effects, single rating, absolute agreement.
#' Note: This combination is rarely used and not recommended.
#'
#' @inheritParams icc_calc_1_1
#' @return A standardized list containing ICC results.
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996).
icc_calc_3_1_absolute <- function(data_matrix, alpha = 0.05, rho0 = NULL, interaction = TRUE) {
  anova_res <- icc_calc_anova(data_matrix, model_type = "twoway", interaction = interaction)
  point_est <- icc_tool_point(anova_res, icc_type = "2,1") # Use same formula as ICC(2,1)
  ci_res <- icc_tool_ci(anova_res, icc_type = "2,1", point_est = point_est, alpha = alpha)
  f_test_null <- icc_calc_f_test(anova_res, icc_type = "2,1", rho0 = 0, alpha = alpha)
  f_test_rho0 <- if (!is.null(rho0)) icc_calc_f_test(anova_res, icc_type = "2,1", rho0 = rho0, alpha = alpha) else NULL

  warning_msg <- paste(
    "WARNING: Fixed effects + absolute agreement combination is rarely used",
    "and difficult to interpret. It is strongly recommended to use",
    "ICC(2,1) (random effects + absolute agreement) instead.",
    "See framework documentation for details."
  )

  .assemble_icc_output(
    icc_type_full = "ICC(3,1,absolute) Two-way mixed, single rating, absolute agreement (NOT RECOMMENDED)",
    icc_type_code = "3,1,absolute",
    anova_res = anova_res,
    point_est = point_est,
    ci_res = ci_res,
    f_test_null = f_test_null,
    f_test_rho0 = f_test_rho0,
    warning_msg = warning_msg,
    tip_msg = NULL
  )
}

#==============================================================================#
# Supplementary ICC: ICC(3,k,absolute)
# Not recommended, provides a warning message
#==============================================================================#
#' Calculate ICC(3,k,absolute) [Supplementary]
#'
#' @description
#' Calculates ICC for two-way mixed effects, average of k ratings, absolute agreement.
#' Note: This combination is rarely used and not recommended.
#'
#' @inheritParams icc_calc_1_1
#' @return A standardized list containing ICC results.
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996).
icc_calc_3_k_absolute <- function(data_matrix, alpha = 0.05, rho0 = NULL, interaction = TRUE) {
  anova_res <- icc_calc_anova(data_matrix, model_type = "twoway", interaction = interaction)
  point_est <- icc_tool_point(anova_res, icc_type = "2,k")
  ci_res <- icc_tool_ci(anova_res, icc_type = "2,k", point_est = point_est, alpha = alpha)
  f_test_null <- icc_calc_f_test(anova_res, icc_type = "2,k", rho0 = 0, alpha = alpha)
  f_test_rho0 <- if (!is.null(rho0)) icc_calc_f_test(anova_res, icc_type = "2,k", rho0 = rho0, alpha = alpha) else NULL

  warning_msg <- paste(
    "WARNING: Fixed effects + absolute agreement combination is rarely used",
    "and difficult to interpret. It is strongly recommended to use",
    "ICC(2,k) (random effects + absolute agreement) instead.",
    "See framework documentation for details."
  )

  .assemble_icc_output(
    icc_type_full = "ICC(3,k,absolute) Two-way mixed, average of k ratings, absolute agreement (NOT RECOMMENDED)",
    icc_type_code = "3,k,absolute",
    anova_res = anova_res,
    point_est = point_est,
    ci_res = ci_res,
    f_test_null = f_test_null,
    f_test_rho0 = f_test_rho0,
    warning_msg = warning_msg,
    tip_msg = NULL
  )
}

# File: utils-data.R
# Description: Data preprocessing and validation for ICC analysis
# Author: [Clare Gao]
# Date: 2026-5-6
# Dependencies: base R (no additional packages required)

#==============================================================================#
# Function 1: icc_preprocess_data
# Core data cleaning, validation and standardization for ICC calculations
#==============================================================================#
#' Preprocess and Validate Data for ICC Analysis
#'
#' @description
#' Performs standardized data cleaning, format conversion, and legality validation
#' for raw input data. Provides a unified valid data input for all ICC calculation
#' functions to avoid repetitive validation code.
#'
#' @param data A data frame or matrix. Rows represent subjects, columns represent raters/
#'   repeated measurements. Must contain only numeric values.
#' @param na.rm Logical. Default is \code{TRUE}. If \code{TRUE}, remove rows with
#'   missing values; if \code{FALSE}, retain missing values and return a warning.
#'
#' @return A named list containing:
#' \describe{
#'   \item{data_matrix}{Numeric matrix. Standardized numeric matrix (no missing values
#'     if \code{na.rm = TRUE}).}
#'   \item{n}{Integer. Number of valid subjects (rows).}
#'   \item{k}{Integer. Number of raters/repeated measurements (columns).}
#'   \item{warning_msg}{Character or \code{NULL}. Warning message for missing values,
#'     \code{NULL} if no missing values.}
#'   \item{error_msg}{Character or \code{NULL}. Error message for invalid data,
#'     \code{NULL} if data is valid.}
#' }
#'
#' @keywords utilities
#' @export
#'
#' @examples
#' # Preprocess the built-in example dataset
#' data(icc_data)
#' processed <- icc_preprocess_data(icc_data)
#' str(processed)

icc_preprocess_data <- function(data, na.rm = TRUE) {
  # 1. Initialize output list with default values
  result <- list(
    data_matrix = NULL,
    n = NULL,
    k = NULL,
    warning_msg = NULL,
    error_msg = NULL
  )

  #----------------------------------------------------------------------------#
  # Step 1: Validate input data type (must be data frame or matrix)
  #----------------------------------------------------------------------------#
  if (!is.data.frame(data) && !is.matrix(data)) {
    result$error_msg <- "Invalid input: 'data' must be a data frame or matrix."
    return(result)
  }

  #----------------------------------------------------------------------------#
  # Step 2: Convert to matrix and validate all values are numeric
  #----------------------------------------------------------------------------#
  data_matrix <- as.matrix(data)
  if (!is.numeric(data_matrix)) {
    result$error_msg <- "Invalid data: All values must be numeric (non-numeric values detected)."
    return(result)
  }

  #----------------------------------------------------------------------------#
  # Step 3: Validate row and column count (minimum 2 rows and 2 columns)
  #----------------------------------------------------------------------------#
  original_rows <- nrow(data_matrix)
  original_cols <- ncol(data_matrix)
  if (original_rows < 2 || original_cols < 2) {
    result$error_msg <- sprintf(
      "Invalid dimensions: Data requires at least 2 subjects (rows) and 2 raters (columns). Current: %d rows, %d columns.",
      original_rows, original_cols
    )
    return(result)
  }

  #----------------------------------------------------------------------------#
  # Step 4: Handle missing values
  #----------------------------------------------------------------------------#
  na_count <- sum(is.na(data_matrix))
  processed_matrix <- data_matrix

  if (na_count > 0) {
    if (na.rm) {
      # Remove rows with missing values
      processed_matrix <- stats::na.omit(processed_matrix)
      valid_rows <- nrow(processed_matrix)

      # Re-validate row count after removing NA
      if (valid_rows < 2) {
        result$error_msg <- sprintf(
          "Insufficient valid data: After removing rows with missing values, only %d valid subject(s) remain (minimum 2 required).",
          valid_rows
        )
        return(result)
      }

      result$warning_msg <- sprintf(
        "Missing values detected: Total %d missing values removed. %d rows deleted, %d valid rows remaining.",
        na_count, original_rows - valid_rows, valid_rows
      )
    } else {
      # Retain NA values and issue warning
      result$warning_msg <- sprintf(
        "Missing values detected: Total %d missing values retained. This may cause errors in ICC calculations.",
        na_count
      )
    }
  }

  #----------------------------------------------------------------------------#
  # Step 5: Populate final results
  #----------------------------------------------------------------------------#
  result$data_matrix <- processed_matrix
  result$n <- nrow(processed_matrix)
  result$k <- ncol(processed_matrix)

  # Return standardized preprocessed result
  return(result)
}

#==============================================================================#
# Function 2: icc_check_design
# Validate core ICC design parameters and generate warnings/tips
#==============================================================================#
#' Validate ICC Design Parameters
#'
#' @description
#' Validates the legality of 4 core user-input design parameters, intercepts
#' invalid inputs in advance, and generates standardized warnings and tips
#' to avoid runtime errors during ICC calculation.
#'
#' @param same_raters Logical. Required. Are all subjects measured by the same
#'   group of raters? TRUE = yes, FALSE = no.
#' @param rater_effect Character. Optional (NULL). Rater effect type: "random" or "fixed".
#'   Not required when \code{same_raters = FALSE}.
#' @param rating_type Character. Required. Type of rating used: "single" or "average".
#' @param agreement_type Character. Optional (NULL). Agreement type: "absolute" or "consistency".
#'   Not required when \code{same_raters = FALSE}.
#' @param k Integer. Required. Number of raters (columns of the data), from \code{icc_preprocess_data}.
#'
#' @return A named list containing:
#' \describe{
#'   \item{is_valid}{Logical. Whether the design parameters are valid.}
#'   \item{error_msg}{Character or NULL. Error message for invalid parameters, NULL if valid.}
#'   \item{warning_msg}{Character or NULL. Warning message for non-recommended scenarios, NULL if none.}
#'   \item{tip_msg}{Character or NULL. Tip message for automatic mapping scenarios, NULL if none.}
#' }
#'
#' @keywords internal
#' @references
#' McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
#' intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
icc_check_design <- function(same_raters, rater_effect = NULL,
                             rating_type, agreement_type = NULL, k) {
  # 1. Initialize output result
  result <- list(
    is_valid = FALSE,
    error_msg = NULL,
    warning_msg = NULL,
    tip_msg = NULL
  )

  #----------------------------------------------------------------------------#
  # Step 1: Basic mandatory parameter validation
  #----------------------------------------------------------------------------#
  # Check same_raters: must be a single logical value
  if (missing(same_raters) || !is.logical(same_raters) || length(same_raters) != 1) {
    result$error_msg <- "Invalid parameter: 'same_raters' must be a single logical value (TRUE/FALSE)."
    return(result)
  }

  # Check rating_type: required, must be single character
  if (missing(rating_type) || !is.character(rating_type) || length(rating_type) != 1) {
    result$error_msg <- "Invalid parameter: 'rating_type' must be a single character: 'single' or 'average'."
    return(result)
  }

  # Check rating_type allowed values
  if (!rating_type %in% c("single", "average")) {
    result$error_msg <- "Invalid 'rating_type': must be either 'single' or 'average'."
    return(result)
  }

  # Check k: must be integer >= 2
  if (missing(k) || !is.numeric(k) || length(k) != 1 || k < 2 || k != as.integer(k)) {
    result$error_msg <- "Invalid parameter: 'k' must be an integer greater than or equal to 2."
    return(result)
  }

  #----------------------------------------------------------------------------#
  # Step 2: One-way model validation (same_raters = FALSE)
  #----------------------------------------------------------------------------#
  if (!same_raters) {
    # For one-way model: rater_effect and agreement_type must NOT be provided
    if (!is.null(rater_effect) || !is.null(agreement_type)) {
      result$error_msg <- "Invalid input: One-way model does NOT require specifying 'rater_effect' or 'agreement_type'."
      return(result)
    }

    # All checks passed for one-way model
    result$is_valid <- TRUE
    return(result)
  }

  #----------------------------------------------------------------------------#
  # Step 3: Two-way model validation (same_raters = TRUE)
  #----------------------------------------------------------------------------#
  if (same_raters) {
    # Check required parameters for two-way model
    if (is.null(rater_effect) || is.null(agreement_type)) {
      result$error_msg <- "Missing parameters: Two-way model requires both 'rater_effect' and 'agreement_type'."
      return(result)
    }

    # Validate rater_effect
    if (!rater_effect %in% c("random", "fixed")) {
      result$error_msg <- "Invalid 'rater_effect': must be either 'random' or 'fixed'."
      return(result)
    }

    # Validate agreement_type
    if (!agreement_type %in% c("absolute", "consistency")) {
      result$error_msg <- "Invalid 'agreement_type': must be either 'absolute' or 'consistency'."
      return(result)
    }

    # All parameter checks passed for two-way model
    result$is_valid <- TRUE

    #--------------------------------------------------------------------------#
    # Generate tip/warning messages for special two-way scenarios
    #--------------------------------------------------------------------------#
    # Tip: Random effects + consistency (auto-mapped to ICC(3,x))
    if (rater_effect == "random" && agreement_type == "consistency") {
      result$tip_msg <- sprintf(
        "Tip: Random effects + consistency combination. Automatically mapped to ICC(3,%s) (mixed effects model) in practice.",
        ifelse(rating_type == "single", "1", "k")
      )
    }

    # Warning: Fixed effects + absolute agreement (not recommended)
    if (rater_effect == "fixed" && agreement_type == "absolute") {
      result$warning_msg <- sprintf(
        "WARNING: Fixed effects + absolute agreement is rarely used and hard to interpret. Recommended: Random effects + absolute agreement (ICC(2,%s)).",
        ifelse(rating_type == "single", "1", "k")
      )
    }
  }

  # Return final validation result
  return(result)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-data.R
\name{icc_check_design}
\alias{icc_check_design}
\title{Validate ICC Design Parameters}
\usage{
icc_check_design(
  same_raters,
  rater_effect = NULL,
  rating_type,
  agreement_type = NULL,
  k
)
}
\arguments{
\item{same_raters}{Logical. Required. Are all subjects measured by the same
group of raters? TRUE = yes, FALSE = no.}

\item{rater_effect}{Character. Optional (NULL). Rater effect type: "random" or "fixed".
Not required when \code{same_raters = FALSE}.}

\item{rating_type}{Character. Required. Type of rating used: "single" or "average".}

\item{agreement_type}{Character. Optional (NULL). Agreement type: "absolute" or "consistency".
Not required when \code{same_raters = FALSE}.}

\item{k}{Integer. Required. Number of raters (columns of the data), from \code{icc_preprocess_data}.}
}
\value{
A named list containing:
\describe{
  \item{is_valid}{Logical. Whether the design parameters are valid.}
  \item{error_msg}{Character or NULL. Error message for invalid parameters, NULL if valid.}
  \item{warning_msg}{Character or NULL. Warning message for non-recommended scenarios, NULL if none.}
  \item{tip_msg}{Character or NULL. Tip message for automatic mapping scenarios, NULL if none.}
}
}
\description{
Validates the legality of 4 core user-input design parameters, intercepts
invalid inputs in advance, and generates standardized warnings and tips
to avoid runtime errors during ICC calculation.
}
\references{
McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-icc.R
\name{icc_calc_3_1_absolute}
\alias{icc_calc_3_1_absolute}
\title{Calculate ICC(3,1,absolute) [Supplementary]}
\usage{
icc_calc_3_1_absolute(
  data_matrix,
  alpha = 0.05,
  rho0 = NULL,
  interaction = TRUE
)
}
\arguments{
\item{data_matrix}{A standardized numeric matrix from \code{icc_preprocess_data}.
Rows = subjects, columns = raters/measurements.}

\item{alpha}{Significance level for confidence interval, default 0.05.}

\item{rho0}{Optional null hypothesis value for non-zero test, default NULL.}

\item{interaction}{Logical, whether to include interaction term (ignored for one-way).}
}
\value{
A standardized list containing ICC results.
}
\description{
Calculates ICC for two-way mixed effects, single rating, absolute agreement.
Note: This combination is rarely used and not recommended.
}
\references{
McGraw, K. O., & Wong, S. P. (1996).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{icc_data}
\alias{icc_data}
\title{Example ICC Dataset}
\format{
A numeric matrix with 5 rows (subjects) and 4 columns (raters).
  Row names are "Subject1" to "Subject5", column names are "Rater1" to "Rater4".
}
\source{
Simulated data for demonstration purposes.
}
\description{
A small, carefully constructed example dataset containing ratings from
4 raters on 5 subjects. Designed to demonstrate all package functionality
with fast execution and predictable results.
}
\details{
This dataset can be used to calculate all 10 ICC types supported by the
package, simply by changing the design parameters in \code{\link{icc_calc}}.


The dataset was simulated to have an approximate ICC of 0.8, which falls
into the "Good" reliability category according to Koo & Li (2016).
}
\examples{
data(icc_data)
head(icc_data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main-interface.R
\name{icc_calc}
\alias{icc_calc}
\title{Calculate Intraclass Correlation Coefficient (ICC)}
\usage{
icc_calc(
  data,
  same_raters,
  rater_effect = NULL,
  rating_type,
  agreement_type = NULL,
  alpha = 0.05,
  rho0 = NULL,
  interaction = TRUE,
  na.rm = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame or matrix. Raw data where rows = subjects,
columns = raters/measurements.}

\item{same_raters}{Logical. Are all subjects measured by the same group
of raters?}

\item{rater_effect}{Character. "random" or "fixed". Ignored if
\code{same_raters = FALSE}.}

\item{rating_type}{Character. "single" (single rating) or "average"
(average of k ratings).}

\item{agreement_type}{Character. "absolute" (absolute agreement) or
"consistency" (consistency). Ignored if \code{same_raters = FALSE}.}

\item{alpha}{Numeric. Significance level for confidence interval,
default 0.05.}

\item{rho0}{Numeric. Optional null hypothesis value for non-zero test,
default NULL.}

\item{interaction}{Logical. Whether to include interaction term in
two-way models, default TRUE.}

\item{na.rm}{Logical. Whether to automatically remove rows with missing
values, default TRUE.}

\item{verbose}{Logical. Whether to emit warnings and tips, default TRUE.}
}
\value{
A named list containing:
\describe{
  \item{data_summary}{List. Data preprocessing summary.}
  \item{icc_result}{List. Full ICC calculation results.}
  \item{evaluation}{List. Reliability evaluation results.}
  \item{report}{Character. Standardized report text.}
  \item{warning_msg}{Character or NULL. Warning message.}
  \item{tip_msg}{Character or NULL. Tip message.}
}
}
\description{
Top-level main function for complete ICC analysis. Users only need to
provide raw data and answer 4 design questions. The function automatically
handles data preprocessing, parameter validation, ICC type mapping,
core calculation, reliability evaluation, and report generation.
}
\examples{
# Example 0: Use built-in example dataset
data(icc_data)
result <- icc_calc(icc_data, same_raters = TRUE, rater_effect = "random",
                   rating_type = "single", agreement_type = "absolute")
# Example 1: One-way random effects, single rating (ICC(1,1))
data <- matrix(rnorm(100), nrow = 20, ncol = 5)
result <- icc_calc(data, same_raters = FALSE, rating_type = "single")

# Example 2: Two-way random effects, average rating, absolute agreement (ICC(2,k))
result <- icc_calc(data, same_raters = TRUE, rater_effect = "random",
                   rating_type = "average", agreement_type = "absolute")

# Example 3: Two-way mixed effects, single rating, consistency (ICC(3,1))
result <- icc_calc(data, same_raters = TRUE, rater_effect = "fixed",
                   rating_type = "single", agreement_type = "consistency")

# Example 4: Special scenario - automatic mapping with tip
# Random effects + consistency (automatically mapped to ICC(3,1))
result <- icc_calc(data, same_raters = TRUE, rater_effect = "random",
                   rating_type = "single", agreement_type = "consistency")

# Example 5: Special scenario - not recommended combination with warning
# Fixed effects + absolute agreement (NOT RECOMMENDED)
result <- icc_calc(data, same_raters = TRUE, rater_effect = "fixed",
                   rating_type = "average", agreement_type = "absolute")

# Example 6: Advanced parameters - custom alpha and non-zero test
result <- icc_calc(data, same_raters = TRUE, rater_effect = "random",
                   rating_type = "single", agreement_type = "absolute",
                   alpha = 0.1, rho0 = 0.6, verbose = FALSE)

# Example 7: Extract specific results
result$icc_result$point_est  # ICC point estimate
result$evaluation$rating_en  # Reliability rating
result$report               # Full text report
}
\references{
McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.

Koo, T. K., & Li, M. Y. (2016). A guideline of selecting and reporting
intraclass correlation coefficients for reliability research. Journal of
Chiropractic Medicine, 15(2), 155-163.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-icc.R
\name{icc_calc_1_k}
\alias{icc_calc_1_k}
\title{Calculate ICC(1,k)}
\usage{
icc_calc_1_k(data_matrix, alpha = 0.05, rho0 = NULL, interaction = TRUE)
}
\arguments{
\item{data_matrix}{A standardized numeric matrix from \code{icc_preprocess_data}.
Rows = subjects, columns = raters/measurements.}

\item{alpha}{Significance level for confidence interval, default 0.05.}

\item{rho0}{Optional null hypothesis value for non-zero test, default NULL.}

\item{interaction}{Logical, whether to include interaction term (ignored for one-way).}
}
\value{
A standardized list containing ICC results.
}
\description{
Calculates the Intraclass Correlation Coefficient (ICC) for a one-way
random effects model using the average of k raters/measurements, focusing
on absolute agreement.
}
\references{
McGraw, K. O., & Wong, S. P. (1996).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_size_calculate.R
\name{icc_sample_size}
\alias{icc_sample_size}
\title{Unified ICC Sample Size & Power Interface}
\usage{
icc_sample_size(method = c("lower", "width", "power"), ...)
}
\arguments{
\item{method}{"lower", "width", "power".}

\item{...}{Arguments passed to underlying functions.}
}
\value{
Sample size or power.
}
\description{
Unified ICC Sample Size & Power Interface
}
\examples{
# Method 1: Sample size based on lower confidence limit (most recommended)
# Ensure 95\% CI lower bound >= 0.75 (good reliability)
n1 <- icc_sample_size(
  method = "lower",
  rho = 0.85,
  rating_target = "good",
  k = 3,
  same_raters = TRUE,
  rater_effect = "random",
  rating_type = "single",
  agreement_type = "absolute"
)

# Method 2: Sample size based on confidence interval width
# Ensure 95\% CI half-width <= 0.1
n2 <- icc_sample_size(
  method = "width",
  rho = 0.7,
  omega = 0.1,
  k = 3,
  same_raters = FALSE,
  rating_type = "average"
)

# Method 3: Power calculation for existing study design
power <- icc_sample_size(
  method = "power",
  n = 30,
  rho = 0.7,
  rho0 = 0.5,
  k = 3,
  same_raters = TRUE,
  rater_effect = "fixed",
  rating_type = "single",
  agreement_type = "consistency"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main-interface.R
\name{icc_map_design_to_icc}
\alias{icc_map_design_to_icc}
\title{Map Design Parameters to ICC Type}
\usage{
icc_map_design_to_icc(same_raters, rater_effect, rating_type, agreement_type)
}
\arguments{
\item{same_raters}{Logical. Are all subjects measured by the same group of raters?}

\item{rater_effect}{Character. "random" or "fixed". Ignored if \code{same_raters = FALSE}.}

\item{rating_type}{Character. "single" (single rating) or "average" (average of k ratings).}

\item{agreement_type}{Character. "absolute" (absolute agreement) or "consistency" (consistency).
Ignored if \code{same_raters = FALSE}.}
}
\value{
A named list containing:
\describe{
  \item{icc_func_name}{Character. Name of the core ICC calculation function.}
  \item{icc_full_name}{Character. Full name of the ICC type.}
  \item{icc_code}{Character. ICC code (e.g., "1,1").}
  \item{warning_msg}{Character or NULL. Warning message for not recommended combinations.}
  \item{tip_msg}{Character or NULL. Tip message for automatically mapped combinations.}
}
}
\description{
Maps user's 4 design questions to the corresponding ICC core calculation
function, full name, and any necessary warnings or tips.
}
\references{
McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-calc.R
\name{icc_tool_point}
\alias{icc_tool_point}
\title{Calculate ICC Point Estimate}
\usage{
icc_tool_point(anova_result, icc_type)
}
\arguments{
\item{anova_result}{List. Output from \code{icc_calc_anova} containing ANOVA statistics.}

\item{icc_type}{Character. ICC type code, must be one of:
"1,1", "1,k", "2,1", "2,k", "3,1", "3,k",
"2,1,consistency", "2,k,consistency", "3,1,absolute", "3,k,absolute"}
}
\value{
Numeric. Point estimate of the intraclass correlation coefficient.
}
\description{
Unified low-level function to compute point estimates for all 10 ICC types
based on the standard formulas from McGraw & Wong (1996) Table 4 and 5.
Serves as the single source of truth for ICC point calculations.
}
\references{
McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_size_calculate.R
\name{icc_sample_size_lower}
\alias{icc_sample_size_lower}
\title{Sample Size for ICC based on Lower Confidence Limit}
\usage{
icc_sample_size_lower(
  rho,
  rho0 = NULL,
  k = 3,
  same_raters,
  rater_effect = NULL,
  rating_type,
  agreement_type = NULL,
  alpha = 0.05,
  assurance = 0.8,
  rating_target = NULL,
  verbose = TRUE
)
}
\arguments{
\item{rho}{Anticipated ICC value.}

\item{rho0}{Desired lower bound.}

\item{k}{Number of observations per subject. Default 3.}

\item{same_raters}{Logical.}

\item{rater_effect}{"random" or "fixed".}

\item{rating_type}{"single" or "average".}

\item{agreement_type}{"absolute" or "consistency".}

\item{alpha}{Significance level. Default 0.05.}

\item{assurance}{Assurance probability. Default 0.8.}

\item{rating_target}{Shortcut for rho0.}

\item{verbose}{Print messages. Default TRUE.}
}
\value{
Required sample size.
}
\description{
Sample Size for ICC based on Lower Confidence Limit
}
\examples{
# Note: It is recommended to use the unified interface icc_sample_size()
icc_sample_size_lower(rho = 0.8, rho0 = 0.6, k = 3, same_raters = FALSE, rating_type = "single")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-calc.R
\name{icc_calc_anova}
\alias{icc_calc_anova}
\title{Calculate ANOVA for ICC Models}
\usage{
icc_calc_anova(data_matrix, model_type, interaction = TRUE)
}
\arguments{
\item{data_matrix}{Numeric matrix. Standardized data matrix from \code{icc_preprocess_data}.}

\item{model_type}{Character. Model type: "oneway" (one-way random) or "twoway" (two-way).}

\item{interaction}{Logical. Default \code{TRUE}. Whether to include subject×rater interaction
term for two-way models (follows standard literature settings). If FALSE, rater effect
is pooled into the residual term.}
}
\value{
A named list containing ANOVA results:
\describe{
  \item{MSR}{Numeric. Mean square for subjects (rows).}
  \item{MSC}{Numeric. Mean square for raters (columns) (NULL for one-way model).}
  \item{MSE}{Numeric. Residual mean square (two-way model only).}
  \item{MSW}{Numeric. Within-group mean square (one-way model only).}
  \item{df1}{Integer. Degrees of freedom for subjects (n-1).}
  \item{df2}{Integer. Residual degrees of freedom.}
  \item{df3}{Integer. Degrees of freedom for raters (k-1) (NULL for one-way model).}
  \item{n}{Integer. Number of subjects.}
  \item{k}{Integer. Number of raters.}
}
}
\description{
Unified function to compute mean squares and degrees of freedom for
one-way random and two-way random/mixed ANOVA models. Serves as the
foundational calculation for all ICC types, eliminating redundant code.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny-app.R
\name{run_icc_app}
\alias{run_icc_app}
\title{Launch the ICCDesign Shiny Application}
\usage{
run_icc_app(
  host = "127.0.0.1",
  port = NULL,
  launch.browser = interactive(),
  ...
)
}
\arguments{
\item{host}{Host address passed to \code{shiny::runApp}. Default is
\code{"127.0.0.1"}.}

\item{port}{Optional port passed to \code{shiny::runApp}. If \code{NULL},
Shiny selects an available port.}

\item{launch.browser}{Logical. Whether to open the app in a browser.
Default is \code{interactive()}.}

\item{...}{Additional arguments passed to \code{shiny::runApp}.}
}
\value{
Runs the Shiny application.
}
\description{
Starts an interactive Shiny application for ICC analysis, reliability
reporting, and sample size or power planning.
}
\examples{
if (interactive()) {
# Launch the interactive Shiny application
run_icc_app()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{ICCDesign-package}
\alias{ICCDesign-package}
\alias{ICCDesign}
\title{ICCDesign: Intraclass Correlation Coefficient Analysis and Study Planning}
\description{
A comprehensive, user-friendly R package for intraclass correlation coefficient (ICC)
analysis and sample size planning. Implements the full McGraw & Wong (1996) framework
supporting all 10 ICC types, with an intuitive 4-question decision system that
eliminates the need for users to memorize ICC type codes.

The package provides both a command-line interface for advanced users and a
fully interactive Shiny web application for point-and-click analysis. It also
includes automated reliability evaluation based on Koo & Li (2016) criteria,
publication-ready report generation, and rigorous sample size and power
calculation functions.
}
\section{Key Features}{

\itemize{
  \item Full support for all 10 ICC types from McGraw & Wong (1996)
  \item Intuitive 4-question decision framework for ICC type selection
  \item Automated reliability rating and interpretation
  \item Publication-ready text, Markdown, and HTML reports
  \item Sample size calculation based on confidence interval lower bound or width
  \item Power analysis for existing study designs
  \item Interactive Shiny web application with data upload and visualization
  \item Comprehensive data validation and error handling
}
}

\section{Main Functions}{

\describe{
  \item{\code{\link{icc_calc}}}{Top-level function for complete ICC analysis}
  \item{\code{\link{icc_sample_size}}}{Unified interface for sample size and power calculation}
  \item{\code{\link{run_icc_app}}}{Launch the interactive Shiny application}
  \item{\code{\link{icc_preprocess_data}}}{Data preprocessing and validation utility}
}
}

\references{
McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.

Koo, T. K., & Li, M. Y. (2016). A guideline of selecting and reporting
intraclass correlation coefficients for reliability research. Journal of
Chiropractic Medicine, 15(2), 155-163.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/KlariZhang/ICCDesign}
  \item Report bugs at \url{https://github.com/KlariZhang/ICCDesign/issues}
}

}
\author{
\strong{Maintainer}: Ziyu Liu \email{1755454769@qq.com}

Authors:
\itemize{
  \item Ziyu Liu \email{1755454769@qq.com}
  \item Ruilin Ma
  \item Chenge Gao
  \item Yundan Zhang
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-data.R
\name{icc_preprocess_data}
\alias{icc_preprocess_data}
\title{Preprocess and Validate Data for ICC Analysis}
\usage{
icc_preprocess_data(data, na.rm = TRUE)
}
\arguments{
\item{data}{A data frame or matrix. Rows represent subjects, columns represent raters/
repeated measurements. Must contain only numeric values.}

\item{na.rm}{Logical. Default is \code{TRUE}. If \code{TRUE}, remove rows with
missing values; if \code{FALSE}, retain missing values and return a warning.}
}
\value{
A named list containing:
\describe{
  \item{data_matrix}{Numeric matrix. Standardized numeric matrix (no missing values
    if \code{na.rm = TRUE}).}
  \item{n}{Integer. Number of valid subjects (rows).}
  \item{k}{Integer. Number of raters/repeated measurements (columns).}
  \item{warning_msg}{Character or \code{NULL}. Warning message for missing values,
    \code{NULL} if no missing values.}
  \item{error_msg}{Character or \code{NULL}. Error message for invalid data,
    \code{NULL} if data is valid.}
}
}
\description{
Performs standardized data cleaning, format conversion, and legality validation
for raw input data. Provides a unified valid data input for all ICC calculation
functions to avoid repetitive validation code.
}
\examples{
# Preprocess the built-in example dataset
data(icc_data)
processed <- icc_preprocess_data(icc_data)
str(processed)
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-calc.R
\name{icc_calc_f_test}
\alias{icc_calc_f_test}
\title{Hypothesis Testing for ICC}
\usage{
icc_calc_f_test(anova_result, icc_type, rho0 = 0, alpha = 0.05)
}
\arguments{
\item{anova_result}{List. Output from \code{icc_calc_anova}.}

\item{icc_type}{Character. ICC type code.}

\item{rho0}{Numeric. Null hypothesis ICC value, default 0.}

\item{alpha}{Numeric. Significance level, default 0.05.}
}
\value{
Named list with test results:
\describe{
  \item{H0}{Character. Null hypothesis statement.}
  \item{F_stat}{Numeric. F test statistic.}
  \item{df1}{Integer. Numerator degrees of freedom.}
  \item{df2}{Integer. Denominator degrees of freedom.}
  \item{p_value}{Numeric. One-tailed p-value (H1: ICC > rho0).}
}
}
\description{
Performs one-tailed F-test for ICC significance following McGraw & Wong (1996) Table 8.
Supports both null hypothesis of zero ICC and custom non-zero null value.
}
\references{
McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main-interface.R
\name{icc_evaluate}
\alias{icc_evaluate}
\title{Evaluate ICC Reliability}
\usage{
icc_evaluate(icc_result)
}
\arguments{
\item{icc_result}{List. Output from a core ICC calculation function.}
}
\value{
A named list with elements:
\itemize{
  \item icc_code: Character. ICC code.
  \item point_est: Numeric. ICC point estimate.
  \item ci_lower: Numeric. 95% CI lower bound.
  \item rating_en: Character. English reliability rating.
  \item explanation: Character. Interpretation text.
}
}
\description{
Evaluates the reliability of an ICC result based on the 95% confidence
interval lower bound, following the criteria of Koo & Li (2016).
}
\references{
Koo, T. K., & Li, M. Y. (2016). A guideline of selecting and reporting
intraclass correlation coefficients for reliability research. Journal of
Chiropractic Medicine, 15(2), 155-163.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main-interface.R
\name{icc_generate_report}
\alias{icc_generate_report}
\title{Generate ICC Report}
\usage{
icc_generate_report(icc_result, evaluation, format = "text")
}
\arguments{
\item{icc_result}{List. Output from a core ICC calculation function.}

\item{evaluation}{List. Output from \code{icc_evaluate}.}

\item{format}{Character. Output format: "text" (default), "markdown", or "html".}
}
\value{
Character. Formatted report text.
}
\description{
Generates a standardized, publication-ready report of ICC results,
supporting text, Markdown, and HTML formats.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny-app.R
\name{icc_shiny_app}
\alias{icc_shiny_app}
\title{Build the ICCDesign Shiny Application}
\usage{
icc_shiny_app()
}
\value{
A Shiny application object.
}
\description{
Build the ICCDesign Shiny Application
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_size_calculate.R
\name{icc_power}
\alias{icc_power}
\title{Power Calculation for ICC Study Design}
\usage{
icc_power(
  n,
  rho,
  rho0 = NULL,
  omega = NULL,
  k = 3,
  same_raters,
  rater_effect = NULL,
  rating_type,
  agreement_type = NULL,
  alpha = 0.05,
  method = c("lower", "width"),
  verbose = TRUE
)
}
\arguments{
\item{n}{Number of subjects.}

\item{rho}{Anticipated ICC value.}

\item{rho0}{Lower bound for method="lower".}

\item{omega}{Half-width for method="width".}

\item{k}{Number of observations. Default 3.}

\item{same_raters}{Logical.}

\item{rater_effect}{"random" or "fixed".}

\item{rating_type}{"single" or "average".}

\item{agreement_type}{"absolute" or "consistency".}

\item{alpha}{Significance level. Default 0.05.}

\item{method}{"lower" or "width".}

\item{verbose}{Print messages. Default TRUE.}
}
\value{
Power.
}
\description{
Power Calculation for ICC Study Design
}
\examples{
# Note: It is recommended to use the unified interface icc_sample_size()
icc_power(n = 30, rho = 0.7, rho0 = 0.5, k = 3,
          same_raters = TRUE, rater_effect = "fixed",
          rating_type = "single", agreement_type = "consistency")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-calc.R
\name{icc_tool_ci}
\alias{icc_tool_ci}
\title{Calculate ICC Confidence Intervals}
\usage{
icc_tool_ci(anova_result, icc_type, point_est, alpha = 0.05)
}
\arguments{
\item{anova_result}{List. Output from \code{icc_calc_anova}.}

\item{icc_type}{Character. ICC type code (matches icc_tool_point).}

\item{point_est}{Numeric. Point estimate from \code{icc_tool_point}.}

\item{alpha}{Numeric. Significance level, default 0.05 (95\% CI).}
}
\value{
A named list with CI results:
\describe{
  \item{ci_level}{Numeric. Confidence level (1 - alpha).}
  \item{ci_lower}{Numeric. Lower bound of CI (truncated to 0).}
  \item{ci_upper}{Numeric. Upper bound of CI (truncated to 1).}
  \item{df_corrected}{Numeric or NULL. Satterthwaite-corrected df (Type A only).}
}
}
\description{
Computes confidence intervals for all 10 ICC types using exact F-distribution
method from McGraw & Wong (1996) Table 7. Implements Satterthwaite degrees of
freedom correction for Type A ICC models (ICC(2,1), ICC(2,k) and their variants).
}
\references{
McGraw, K. O., & Wong, S. P. (1996). Forming inferences about some
intraclass correlation coefficients. Psychological Methods, 1(1), 30-46.
}
\keyword{internal}
